package it.cnr.isti.epasmed.config;

import java.io.File;
import java.nio.charset.Charset;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.sql.DataSource;

import org.springframework.beans.factory.BeanClassLoaderAware;
import org.springframework.beans.factory.BeanCreationException;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.boot.jdbc.DataSourceInitializationMode;
import org.springframework.boot.jdbc.DatabaseDriver;
import org.springframework.boot.jdbc.EmbeddedDatabaseConnection;
import org.springframework.util.Assert;
import org.springframework.util.ClassUtils;
import org.springframework.util.StringUtils;

/**
 * Properties specific to Epasmed.
 * <p>
 * Properties are configured in the {@code application.yml} file. See
 * {@link io.github.jhipster.config.JHipsterProperties} for a good example.
 */
@ConfigurationProperties(prefix = "application", ignoreUnknownFields = false)
public class ApplicationProperties {

	private final DatasourceEpasmed datasourceEpasmed = new DatasourceEpasmed();
	private final DatasourceSistemainformativo datasourceSistemainformativo = new DatasourceSistemainformativo();
	private final DatasourceEpasRest datasourceEpasRest = new DatasourceEpasRest();

	public static class DatasourceEpasmed {
		private final DataSourceProperties datasource = new DataSourceProperties();
		private final LiquibaseProperties liquibase = new LiquibaseProperties();
		private final HikariProperties hikari = new HikariProperties();

		public DataSourceProperties getDatasource() {
			return datasource;
		}

		public LiquibaseProperties getLiquibase() {
			return liquibase;
		}

		public HikariProperties getHikari() {
			return hikari;
		}

	}

	public static class DatasourceSistemainformativo {
		private final DataSourceProperties datasource = new DataSourceProperties();
		private final HikariProperties hikari = new HikariProperties();

		public DataSourceProperties getDatasource() {
			return datasource;
		}

		public HikariProperties getHikari() {
			return hikari;
		}

	}

	public static class DatasourceEpasRest {
		private String restUrl;
		private String restUsername1;
		private String restPassword1;
		private String restUsername2;
		private String restPassword2;
		private String restUsername3;
		private String restPassword3;

		public String getRestUrl() {
			return restUrl;
		}

		public void setRestUrl(String restUrl) {
			this.restUrl = restUrl;
		}

		public String getRestUsername1() {
			return restUsername1;
		}

		public void setRestUsername1(String restUsername1) {
			this.restUsername1 = restUsername1;
		}

		public String getRestPassword1() {
			return restPassword1;
		}

		public void setRestPassword1(String restPassword1) {
			this.restPassword1 = restPassword1;
		}

		public String getRestUsername2() {
			return restUsername2;
		}

		public void setRestUsername2(String restUsername2) {
			this.restUsername2 = restUsername2;
		}

		public String getRestPassword2() {
			return restPassword2;
		}

		public void setRestPassword2(String restPassword2) {
			this.restPassword2 = restPassword2;
		}

		public String getRestUsername3() {
			return restUsername3;
		}

		public void setRestUsername3(String restUsername3) {
			this.restUsername3 = restUsername3;
		}

		public String getRestPassword3() {
			return restPassword3;
		}

		public void setRestPassword3(String restPassword3) {
			this.restPassword3 = restPassword3;
		}

		@Override
		public String toString() {
			return "DatasourceEpasRest [restUrl=" + restUrl + ", restUsername1=" + restUsername1 + ", restPassword1="
					+ restPassword1 + ", restUsername2=" + restUsername2 + ", restPassword2=" + restPassword2
					+ ", restUsername3=" + restUsername3 + ", restPassword3=" + restPassword3 + "]";
		}

		
	}

	public static class HikariProperties {
		private String jdbc_url;
		private String database_username;
		private String database_password;
		private boolean cachePrepStmts = true;
		private int prepStmtCacheSize = 250;
		private int prepStmtCacheSqlLimit = 2048;
		private String poolName = "Hikari1";
		private boolean autoCommit = false;
		private int maximumPoolSize = 10;

		public String getPoolName() {
			return poolName;
		}

		public void setPoolName(String poolName) {
			this.poolName = poolName;
		}

		public boolean isAutoCommit() {
			return autoCommit;
		}

		public void setAutoCommit(boolean autoCommit) {
			this.autoCommit = autoCommit;
		}

		public int getMaximumPoolSize() {
			return maximumPoolSize;
		}

		public void setMaximumPoolSize(int maximumPoolSize) {
			this.maximumPoolSize = maximumPoolSize;
		}

		public String getJdbc_url() {
			return jdbc_url;
		}

		public String getDatabase_username() {
			return database_username;
		}

		public void setDatabase_username(String database_username) {
			this.database_username = database_username;
		}

		public String getDatabase_password() {
			return database_password;
		}

		public void setDatabase_password(String database_password) {
			this.database_password = database_password;
		}

		public boolean isCachePrepStmts() {
			return cachePrepStmts;
		}

		public void setCachePrepStmts(boolean cachePrepStmts) {
			this.cachePrepStmts = cachePrepStmts;
		}

		public int getPrepStmtCacheSize() {
			return prepStmtCacheSize;
		}

		public void setPrepStmtCacheSize(int prepStmtCacheSize) {
			this.prepStmtCacheSize = prepStmtCacheSize;
		}

		public int getPrepStmtCacheSqlLimit() {
			return prepStmtCacheSqlLimit;
		}

		public void setPrepStmtCacheSqlLimit(int prepStmtCacheSqlLimit) {
			this.prepStmtCacheSqlLimit = prepStmtCacheSqlLimit;
		}

		public void setJdbc_url(String jdbc_url) {
			this.jdbc_url = jdbc_url;
		}

		@Override
		public String toString() {
			return "HikariProperties [jdbc_url=" + jdbc_url + ", database_username=" + database_username
					+ ", database_password=" + database_password + ", cachePrepStmts=" + cachePrepStmts
					+ ", prepStmtCacheSize=" + prepStmtCacheSize + ", prepStmtCacheSqlLimit=" + prepStmtCacheSqlLimit
					+ ", poolName=" + poolName + ", autoCommit=" + autoCommit + ", maximumPoolSize=" + maximumPoolSize
					+ "]";
		}

	}

	public static class DataSourceProperties implements BeanClassLoaderAware, InitializingBean {

		private ClassLoader classLoader;

		/**
		 * Name of the datasource. Default to "testdb" when using an embedded database.
		 */
		private String name;

		/**
		 * Whether to generate a random datasource name.
		 */
		private boolean generateUniqueName;

		/**
		 * Fully qualified name of the connection pool implementation to use. By
		 * default, it is auto-detected from the classpath.
		 */
		private Class<? extends DataSource> type;

		/**
		 * Fully qualified name of the JDBC driver. Auto-detected based on the URL by
		 * default.
		 */
		private String driverClassName;

		/**
		 * JDBC URL of the database.
		 */
		private String url;

		/**
		 * Login username of the database.
		 */
		private String username;

		/**
		 * Login password of the database.
		 */
		private String password;

		/**
		 * JNDI location of the datasource. Class, url, username & password are ignored
		 * when set.
		 */
		private String jndiName;

		/**
		 * Initialize the datasource with available DDL and DML scripts.
		 */
		private DataSourceInitializationMode initializationMode = DataSourceInitializationMode.EMBEDDED;

		/**
		 * Platform to use in the DDL or DML scripts (such as schema-${platform}.sql or
		 * data-${platform}.sql).
		 */
		private String platform = "all";

		/**
		 * Schema (DDL) script resource references.
		 */
		private List<String> schema;

		/**
		 * Username of the database to execute DDL scripts (if different).
		 */
		private String schemaUsername;

		/**
		 * Password of the database to execute DDL scripts (if different).
		 */
		private String schemaPassword;

		/**
		 * Data (DML) script resource references.
		 */
		private List<String> data;

		/**
		 * Username of the database to execute DML scripts (if different).
		 */
		private String dataUsername;

		/**
		 * Password of the database to execute DML scripts (if different).
		 */
		private String dataPassword;

		/**
		 * Whether to stop if an error occurs while initializing the database.
		 */
		private boolean continueOnError = false;

		/**
		 * Statement separator in SQL initialization scripts.
		 */
		private String separator = ";";

		/**
		 * SQL scripts encoding.
		 */
		private Charset sqlScriptEncoding;

		private EmbeddedDatabaseConnection embeddedDatabaseConnection = EmbeddedDatabaseConnection.NONE;

		private Xa xa = new Xa();

		private String uniqueName;

		@Override
		public void setBeanClassLoader(ClassLoader classLoader) {
			this.classLoader = classLoader;
		}

		@Override
		public void afterPropertiesSet() throws Exception {
			this.embeddedDatabaseConnection = EmbeddedDatabaseConnection.get(this.classLoader);
		}

		/**
		 * Initialize a {@link DataSourceBuilder} with the state of this instance.
		 * 
		 * @return a {@link DataSourceBuilder} initialized with the customizations
		 *         defined on this instance
		 */
		public DataSourceBuilder<?> initializeDataSourceBuilder() {
			return DataSourceBuilder.create(getClassLoader()).type(getType())
					.driverClassName(determineDriverClassName()).url(determineUrl()).username(determineUsername())
					.password(determinePassword());
		}

		public String getName() {
			return this.name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public boolean isGenerateUniqueName() {
			return this.generateUniqueName;
		}

		public void setGenerateUniqueName(boolean generateUniqueName) {
			this.generateUniqueName = generateUniqueName;
		}

		public Class<? extends DataSource> getType() {
			return this.type;
		}

		public void setType(Class<? extends DataSource> type) {
			this.type = type;
		}

		/**
		 * Return the configured driver or {@code null} if none was configured.
		 * 
		 * @return the configured driver
		 * @see #determineDriverClassName()
		 */
		public String getDriverClassName() {
			return this.driverClassName;
		}

		public void setDriverClassName(String driverClassName) {
			this.driverClassName = driverClassName;
		}

		/**
		 * Determine the driver to use based on this configuration and the environment.
		 * 
		 * @return the driver to use
		 * @since 1.4.0
		 */
		public String determineDriverClassName() {
			if (StringUtils.hasText(this.driverClassName)) {
				Assert.state(driverClassIsLoadable(), () -> "Cannot load driver class: " + this.driverClassName);
				return this.driverClassName;
			}
			String driverClassName = null;
			if (StringUtils.hasText(this.url)) {
				driverClassName = DatabaseDriver.fromJdbcUrl(this.url).getDriverClassName();
			}
			if (!StringUtils.hasText(driverClassName)) {
				driverClassName = this.embeddedDatabaseConnection.getDriverClassName();
			}
			if (!StringUtils.hasText(driverClassName)) {
				throw new DataSourceBeanCreationException("Failed to determine a suitable driver class", this,
						this.embeddedDatabaseConnection);
			}
			return driverClassName;
		}

		private boolean driverClassIsLoadable() {
			try {
				ClassUtils.forName(this.driverClassName, null);
				return true;
			} catch (UnsupportedClassVersionError ex) {
				// Driver library has been compiled with a later JDK, propagate error
				throw ex;
			} catch (Throwable ex) {
				return false;
			}
		}

		/**
		 * Return the configured url or {@code null} if none was configured.
		 * 
		 * @return the configured url
		 * @see #determineUrl()
		 */
		public String getUrl() {
			return this.url;
		}

		public void setUrl(String url) {
			this.url = url;
		}

		/**
		 * Determine the url to use based on this configuration and the environment.
		 * 
		 * @return the url to use
		 * @since 1.4.0
		 */
		public String determineUrl() {
			if (StringUtils.hasText(this.url)) {
				return this.url;
			}
			String databaseName = determineDatabaseName();
			String url = (databaseName != null) ? this.embeddedDatabaseConnection.getUrl(databaseName) : null;
			if (!StringUtils.hasText(url)) {
				throw new DataSourceBeanCreationException("Failed to determine suitable jdbc url", this,
						this.embeddedDatabaseConnection);
			}
			return url;
		}

		/**
		 * Determine the name to used based on this configuration.
		 * 
		 * @return the database name to use or {@code null}
		 * @since 2.0.0
		 */
		public String determineDatabaseName() {
			if (this.generateUniqueName) {
				if (this.uniqueName == null) {
					this.uniqueName = UUID.randomUUID().toString();
				}
				return this.uniqueName;
			}
			if (StringUtils.hasLength(this.name)) {
				return this.name;
			}
			if (this.embeddedDatabaseConnection != EmbeddedDatabaseConnection.NONE) {
				return "testdb";
			}
			return null;
		}

		/**
		 * Return the configured username or {@code null} if none was configured.
		 * 
		 * @return the configured username
		 * @see #determineUsername()
		 */
		public String getUsername() {
			return this.username;
		}

		public void setUsername(String username) {
			this.username = username;
		}

		/**
		 * Determine the username to use based on this configuration and the
		 * environment.
		 * 
		 * @return the username to use
		 * @since 1.4.0
		 */
		public String determineUsername() {
			if (StringUtils.hasText(this.username)) {
				return this.username;
			}
			if (EmbeddedDatabaseConnection.isEmbedded(determineDriverClassName())) {
				return "sa";
			}
			return null;
		}

		/**
		 * Return the configured password or {@code null} if none was configured.
		 * 
		 * @return the configured password
		 * @see #determinePassword()
		 */
		public String getPassword() {
			return this.password;
		}

		public void setPassword(String password) {
			this.password = password;
		}

		/**
		 * Determine the password to use based on this configuration and the
		 * environment.
		 * 
		 * @return the password to use
		 * @since 1.4.0
		 */
		public String determinePassword() {
			if (StringUtils.hasText(this.password)) {
				return this.password;
			}
			if (EmbeddedDatabaseConnection.isEmbedded(determineDriverClassName())) {
				return "";
			}
			return null;
		}

		public String getJndiName() {
			return this.jndiName;
		}

		/**
		 * Allows the DataSource to be managed by the container and obtained via JNDI.
		 * The {@code URL}, {@code driverClassName}, {@code username} and
		 * {@code password} fields will be ignored when using JNDI lookups.
		 * 
		 * @param jndiName the JNDI name
		 */
		public void setJndiName(String jndiName) {
			this.jndiName = jndiName;
		}

		public DataSourceInitializationMode getInitializationMode() {
			return this.initializationMode;
		}

		public void setInitializationMode(DataSourceInitializationMode initializationMode) {
			this.initializationMode = initializationMode;
		}

		public String getPlatform() {
			return this.platform;
		}

		public void setPlatform(String platform) {
			this.platform = platform;
		}

		public List<String> getSchema() {
			return this.schema;
		}

		public void setSchema(List<String> schema) {
			this.schema = schema;
		}

		public String getSchemaUsername() {
			return this.schemaUsername;
		}

		public void setSchemaUsername(String schemaUsername) {
			this.schemaUsername = schemaUsername;
		}

		public String getSchemaPassword() {
			return this.schemaPassword;
		}

		public void setSchemaPassword(String schemaPassword) {
			this.schemaPassword = schemaPassword;
		}

		public List<String> getData() {
			return this.data;
		}

		public void setData(List<String> data) {
			this.data = data;
		}

		public String getDataUsername() {
			return this.dataUsername;
		}

		public void setDataUsername(String dataUsername) {
			this.dataUsername = dataUsername;
		}

		public String getDataPassword() {
			return this.dataPassword;
		}

		public void setDataPassword(String dataPassword) {
			this.dataPassword = dataPassword;
		}

		public boolean isContinueOnError() {
			return this.continueOnError;
		}

		public void setContinueOnError(boolean continueOnError) {
			this.continueOnError = continueOnError;
		}

		public String getSeparator() {
			return this.separator;
		}

		public void setSeparator(String separator) {
			this.separator = separator;
		}

		public Charset getSqlScriptEncoding() {
			return this.sqlScriptEncoding;
		}

		public void setSqlScriptEncoding(Charset sqlScriptEncoding) {
			this.sqlScriptEncoding = sqlScriptEncoding;
		}

		public ClassLoader getClassLoader() {
			return this.classLoader;
		}

		public Xa getXa() {
			return this.xa;
		}

		public void setXa(Xa xa) {
			this.xa = xa;
		}

		/**
		 * XA Specific datasource settings.
		 */
		public class Xa {

			/**
			 * XA datasource fully qualified name.
			 */
			private String dataSourceClassName;

			/**
			 * Properties to pass to the XA data source.
			 */
			private Map<String, String> properties = new LinkedHashMap<>();

			public String getDataSourceClassName() {
				return this.dataSourceClassName;
			}

			public void setDataSourceClassName(String dataSourceClassName) {
				this.dataSourceClassName = dataSourceClassName;
			}

			public Map<String, String> getProperties() {
				return this.properties;
			}

			public void setProperties(Map<String, String> properties) {
				this.properties = properties;
			}

		}

		public class DataSourceBeanCreationException extends BeanCreationException {

			private static final long serialVersionUID = 1L;

			private final DataSourceProperties properties;

			private final EmbeddedDatabaseConnection connection;

			DataSourceBeanCreationException(String message, DataSourceProperties properties,
					EmbeddedDatabaseConnection connection) {
				super(message);
				this.properties = properties;
				this.connection = connection;
			}

			DataSourceProperties getProperties() {
				return this.properties;
			}

			EmbeddedDatabaseConnection getConnection() {
				return this.connection;
			}

		}

	}

	public static class LiquibaseProperties {

		/**
		 * Change log configuration path.
		 */
		private String changeLog = "classpath:/db/changelog/db.changelog-master.yaml";

		/**
		 * Comma-separated list of runtime contexts to use.
		 */
		private String contexts;

		/**
		 * Default database schema.
		 */
		private String defaultSchema;

		/**
		 * Schema to use for Liquibase objects.
		 */
		private String liquibaseSchema;

		/**
		 * Tablespace to use for Liquibase objects.
		 */
		private String liquibaseTablespace;

		/**
		 * Name of table to use for tracking change history.
		 */
		private String databaseChangeLogTable = "DATABASECHANGELOG";

		/**
		 * Name of table to use for tracking concurrent Liquibase usage.
		 */
		private String databaseChangeLogLockTable = "DATABASECHANGELOGLOCK";

		/**
		 * Whether to first drop the database schema.
		 */
		private boolean dropFirst;

		/**
		 * Whether to enable Liquibase support.
		 */
		private boolean enabled = true;

		/**
		 * Login user of the database to migrate.
		 */
		private String user;

		/**
		 * Login password of the database to migrate.
		 */
		private String password;

		/**
		 * JDBC URL of the database to migrate. If not set, the primary configured data
		 * source is used.
		 */
		private String url;

		/**
		 * Comma-separated list of runtime labels to use.
		 */
		private String labels;

		/**
		 * Change log parameters.
		 */
		private Map<String, String> parameters;

		/**
		 * File to which rollback SQL is written when an update is performed.
		 */
		private File rollbackFile;

		/**
		 * Whether rollback should be tested before update is performed.
		 */
		private boolean testRollbackOnUpdate;

		public String getChangeLog() {
			return this.changeLog;
		}

		public void setChangeLog(String changeLog) {
			Assert.notNull(changeLog, "ChangeLog must not be null");
			this.changeLog = changeLog;
		}

		public String getContexts() {
			return this.contexts;
		}

		public void setContexts(String contexts) {
			this.contexts = contexts;
		}

		public String getDefaultSchema() {
			return this.defaultSchema;
		}

		public void setDefaultSchema(String defaultSchema) {
			this.defaultSchema = defaultSchema;
		}

		public String getLiquibaseSchema() {
			return this.liquibaseSchema;
		}

		public void setLiquibaseSchema(String liquibaseSchema) {
			this.liquibaseSchema = liquibaseSchema;
		}

		public String getLiquibaseTablespace() {
			return this.liquibaseTablespace;
		}

		public void setLiquibaseTablespace(String liquibaseTablespace) {
			this.liquibaseTablespace = liquibaseTablespace;
		}

		public String getDatabaseChangeLogTable() {
			return this.databaseChangeLogTable;
		}

		public void setDatabaseChangeLogTable(String databaseChangeLogTable) {
			this.databaseChangeLogTable = databaseChangeLogTable;
		}

		public String getDatabaseChangeLogLockTable() {
			return this.databaseChangeLogLockTable;
		}

		public void setDatabaseChangeLogLockTable(String databaseChangeLogLockTable) {
			this.databaseChangeLogLockTable = databaseChangeLogLockTable;
		}

		public boolean isDropFirst() {
			return this.dropFirst;
		}

		public void setDropFirst(boolean dropFirst) {
			this.dropFirst = dropFirst;
		}

		public boolean isEnabled() {
			return this.enabled;
		}

		public void setEnabled(boolean enabled) {
			this.enabled = enabled;
		}

		public String getUser() {
			return this.user;
		}

		public void setUser(String user) {
			this.user = user;
		}

		public String getPassword() {
			return this.password;
		}

		public void setPassword(String password) {
			this.password = password;
		}

		public String getUrl() {
			return this.url;
		}

		public void setUrl(String url) {
			this.url = url;
		}

		public String getLabels() {
			return this.labels;
		}

		public void setLabels(String labels) {
			this.labels = labels;
		}

		public Map<String, String> getParameters() {
			return this.parameters;
		}

		public void setParameters(Map<String, String> parameters) {
			this.parameters = parameters;
		}

		public File getRollbackFile() {
			return this.rollbackFile;
		}

		public void setRollbackFile(File rollbackFile) {
			this.rollbackFile = rollbackFile;
		}

		public boolean isTestRollbackOnUpdate() {
			return this.testRollbackOnUpdate;
		}

		public void setTestRollbackOnUpdate(boolean testRollbackOnUpdate) {
			this.testRollbackOnUpdate = testRollbackOnUpdate;
		}

	}

	public DatasourceEpasmed getDatasourceEpasmed() {
		return datasourceEpasmed;
	}

	public DatasourceSistemainformativo getDatasourceSistemainformativo() {
		return datasourceSistemainformativo;
	}

	public DatasourceEpasRest getDatasourceEpasRest() {
		return datasourceEpasRest;
	}
}
