package it.cnr.isti.epasmed.config;

import javax.sql.DataSource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.jdbc.repository.config.AbstractJdbcConfiguration;
import org.springframework.data.jdbc.repository.config.EnableJdbcRepositories;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcOperations;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.transaction.TransactionManager;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;

import it.cnr.isti.epasmed.config.ApplicationProperties.DataSourceProperties;
import it.cnr.isti.epasmed.config.ApplicationProperties.DatasourceSistemainformativo;
import it.cnr.isti.epasmed.config.ApplicationProperties.HikariProperties;

@Configuration
@EnableJdbcRepositories(basePackages = "it.cnr.isti.epasmed.sistemainformativo.repository")
//@EnableJpaRepositories(basePackages = "it.cnr.isti.epasmed.sistemainformativo.repository", entityManagerFactoryRef = "sistemaInformativoEntityManagerFactory", transactionManagerRef = "sistemaInformativoTransactionManager")
//@EnableJpaAuditing(auditorAwareRef = "springSecurityAuditorAware")
@EnableTransactionManagement
public class SistemaInformativoDataSourceConfiguration extends AbstractJdbcConfiguration {

	private final Logger log = LoggerFactory.getLogger(SistemaInformativoDataSourceConfiguration.class);

	private final ApplicationProperties applicationProperties;

	public SistemaInformativoDataSourceConfiguration(ApplicationProperties applicationProperties) {
		this.applicationProperties = applicationProperties;
	}

	@Bean
	public DatasourceSistemainformativo sistemaInformativoDataSourceProperties() {
		return applicationProperties.getDatasourceSistemainformativo();
	}

	@Bean(name = "sistemaInformativoDataSource")
	public DataSource sistemaInformativoDataSource() {
		DataSourceProperties dsp = sistemaInformativoDataSourceProperties().getDatasource();
		HikariProperties hikariProperties  = sistemaInformativoDataSourceProperties().getHikari();
		
		//DriverManagerDataSource dataSource = new DriverManagerDataSource();
		//dataSource.setDriverClassName(dsp.getDriverClassName());
		//dataSource.setUrl(dsp.getUrl());
		//dataSource.setUsername(dsp.getUsername());
		//dataSource.setPassword(dsp.getPassword());
		//log.info("SistemaInformativoDataSource created");
		
		
		HikariConfig config = new HikariConfig();
		config.setJdbcUrl(dsp.getUrl());
		config.setDriverClassName(dsp.getDriverClassName());
		config.setUsername(dsp.getUsername());
		config.setPassword(dsp.getPassword());
		config.addDataSourceProperty("cachePrepStmts", hikariProperties.isCachePrepStmts());
		config.addDataSourceProperty("prepStmtCacheSize", hikariProperties.getPrepStmtCacheSize());
		config.addDataSourceProperty("prepStmtCacheSqlLimit", hikariProperties.getPrepStmtCacheSqlLimit());
		config.setAutoCommit(hikariProperties.isAutoCommit());
		config.setPoolName(hikariProperties.getPoolName());
		config.setMaximumPoolSize(hikariProperties.getMaximumPoolSize());
		log.debug("HikariConfig sistemainformativo: " + dsp.getUrl());
		HikariDataSource dataSource = new HikariDataSource(config);
		return dataSource;
	}

	

	@Bean
	NamedParameterJdbcOperations namedParameterJdbcOperations(
			final @Qualifier("sistemaInformativoDataSource") DataSource dataSource) {
		return new NamedParameterJdbcTemplate(dataSource);
	}

	@Bean
	TransactionManager sistemaInformativoTransactionManager(
			final @Qualifier("sistemaInformativoDataSource") DataSource dataSource) {
		return new DataSourceTransactionManager(dataSource);
	}
}
