package it.cnr.isti.epasmed.config;

import java.time.LocalDateTime;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import it.cnr.isti.epasmed.sync.SyncService;

@Configuration
@EnableScheduling
public class SyncConfiguration {

	private final Logger logger = LoggerFactory.getLogger(SyncConfiguration.class);

	private final SyncService syncService;

	public SyncConfiguration(SyncService syncService) {
		this.syncService = syncService;

	}

	@Scheduled(cron = "0 50 7 * * ?")
	public void cronJobSyncRead() {

		LocalDateTime start = LocalDateTime.now();
		logger.info("Scheduled Sync Read Start : {}", start);

		try {
			syncService.executeReads();
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
		}

		LocalDateTime end = LocalDateTime.now();
		logger.info("Scheduled Sync Read End : {}", end);

	}
	
	@Scheduled(cron = "0 0 3 * * ?")
	public void cronJobSyncWrite() {

		LocalDateTime start = LocalDateTime.now();
		logger.info("Scheduled Sync Write Start : {}", start);
		String year=String.valueOf(start.getYear());
		String month=String.valueOf(start.getMonthValue());
		
		
		try {
			syncService.executeWritesScheduled(year, month);
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
		}

		LocalDateTime end = LocalDateTime.now();
		logger.info("Scheduled Sync Write End : {}", end);

	}
	
}
