package it.cnr.isti.epasmed.epas.client;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import it.cnr.isti.epasmed.config.ApplicationProperties;
import it.cnr.isti.epasmed.epas.dto.EPASAffiliationsDTO;
import it.cnr.isti.epasmed.epas.model.EPASAffiliations;

@Component
public class EPASAffiliationsClient {

	private final Logger log = LoggerFactory.getLogger(getClass());

	@Autowired
	@Qualifier("RestTemplateForFirstUser")
	RestTemplate rt;

	@Autowired
	ApplicationProperties appProps;

	public EPASAffiliations getById(String id) {
		EPASAffiliations epasAffiliations = rt.getForObject(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/affiliations/show?id={id}", EPASAffiliations.class,
				id);
		log.info("Retrieved Affiliations: {}", epasAffiliations);
		return epasAffiliations;
	}

	public List<EPASAffiliations> getByGroup(String groupId, boolean includeInactive) {
		ResponseEntity<List<EPASAffiliations>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl()
						+ "/v2/affiliations/byGroup?id={groupId}&includeInactive={includeInactive}",
				HttpMethod.GET, null, new ParameterizedTypeReference<List<EPASAffiliations>>() {
				}, groupId,includeInactive);
		List<EPASAffiliations> listEPASAffiliations = responseEntity.getBody();
		log.info("Retrieved Affiliations by group id: {}", listEPASAffiliations);
		return listEPASAffiliations;
	}

	public List<EPASAffiliations> getByPersonId(String personId) {
		ResponseEntity<List<EPASAffiliations>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl()
						+ "/v2/affiliations/byPerson?id={personId}",
				HttpMethod.GET, null, new ParameterizedTypeReference<List<EPASAffiliations>>() {
				}, personId);
		List<EPASAffiliations> listEPASAffiliations = responseEntity.getBody();
		log.info("Retrieved Affiliations by person id: {}", listEPASAffiliations);
		return listEPASAffiliations;
	}
	
	public List<EPASAffiliations> getByPersonFiscalcode(String fc) {
		ResponseEntity<List<EPASAffiliations>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl()
						+ "/v2/affiliations/byPerson?fiscalCode={fc}",
				HttpMethod.GET, null, new ParameterizedTypeReference<List<EPASAffiliations>>() {
				}, fc);
		List<EPASAffiliations> listEPASAffiliations = responseEntity.getBody();
		log.info("Retrieved Affiliations by person fiscalcode: {}", listEPASAffiliations);
		return listEPASAffiliations;
	}

	
	
	public EPASAffiliations create(EPASAffiliationsDTO epasAffiliationsDTO) {
		ResponseEntity<EPASAffiliations> response = rt.postForEntity(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/affiliations/create", epasAffiliationsDTO,
				EPASAffiliations.class);

		EPASAffiliations createdEPASAffiliations = response.getBody();

		log.info("Created Affiliations: {}", createdEPASAffiliations);
		return createdEPASAffiliations;
	}

	public void updateById(String id, @Valid EPASAffiliationsDTO epasAffiliationsDTO) {
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("id", id);

		rt.put(appProps.getDatasourceEpasRest().getRestUrl() + "/v2/affiliations/update?id={id}", epasAffiliationsDTO,
				uriVariables);

		log.info("Updated Affiliations id: {}", id);
		return;
	}

	public void deleteById(String id) {
		rt.delete(appProps.getDatasourceEpasRest().getRestUrl() + "/v2/affiliations/delete?id={id}", id);
		log.info("Deleted Group id: {}", id);
		return;
	}



}
