package it.cnr.isti.epasmed.epas.client;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import it.cnr.isti.epasmed.config.ApplicationProperties;
import it.cnr.isti.epasmed.epas.model.EPASCertifications;

@Component
public class EPASCertificationsClient {

	private final Logger log = LoggerFactory.getLogger(getClass());

	@Autowired
	@Qualifier("RestTemplateForSecondUser")
	RestTemplate rt;

	@Autowired
	ApplicationProperties appProps;

	public EPASCertifications getByPersonId(String id, String year, String month) {
		log.info("Retrieving EPASCertifications by person id: {}", id);
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("id", id);
		uriVariables.put("year", year);
		uriVariables.put("month", month);
		
		EPASCertifications epasCertifications = rt.getForObject(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/certifications/getMonthSituation?id={id}&year={year}&month={month}", 
				EPASCertifications.class,uriVariables);
		log.info("Retrieved EPASCertifications: {}", epasCertifications);
		return epasCertifications;
	}

	public EPASCertifications getByPersonFiscalCode(String fc, String year, String month) {
		log.info("Retrieving EPASCertifications by person fiscalcode: {}", fc);
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("fc", fc);
		uriVariables.put("year", year);
		uriVariables.put("month", month);
		
		EPASCertifications epasCertifications = rt.getForObject(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/certifications/getMonthSituation?fiscalCode={fc}&year={year}&month={month}", 
				EPASCertifications.class,uriVariables);
		log.info("Retrieved EPASCertifications: {}", epasCertifications);
		return epasCertifications;
	}

	public EPASCertifications getByPersonEmail(String email, String year, String month) {
		log.info("Retrieving EPASCertifications by person email={}, year={}, month={}", email,year,month);
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("email", email);
		uriVariables.put("year", year);
		uriVariables.put("month", month);
		
		EPASCertifications epasCertifications = rt.getForObject(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/certifications/getMonthSituation?email={email}&year={year}&month={month}", 
				EPASCertifications.class,uriVariables);
		log.info("Retrieved EPASCertifications: {}", epasCertifications);
		return epasCertifications;
	}

	
	
	public List<EPASCertifications> getByOfficeCodeId(String id, String year, String month) {
		log.info("Retrieving EPASCertifications by office codeId: {}", id);
		
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("sedeId", id);
		uriVariables.put("year", year);
		uriVariables.put("month", month);
		
		ResponseEntity<List<EPASCertifications>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/certifications/getMonthSituationByOffice?sedeId={sedeId}&year={year}&month={month}", HttpMethod.GET, null,
				new ParameterizedTypeReference<List<EPASCertifications>>() {
				}, uriVariables);
		List<EPASCertifications> listEPASTimeCards = responseEntity.getBody();
		log.info("Retrieved EPASCertifications: {}", listEPASTimeCards);
		return listEPASTimeCards;
	}
	
}
