package it.cnr.isti.epasmed.epas.client;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import it.cnr.isti.epasmed.config.ApplicationProperties;
import it.cnr.isti.epasmed.epas.dto.EPASChildrenDTO;
import it.cnr.isti.epasmed.epas.model.EPASChildren;

@Component
public class EPASChildrenClient {

	private final Logger log = LoggerFactory.getLogger(getClass());

	@Autowired
	@Qualifier("RestTemplateForFirstUser")
	RestTemplate rt;

	@Autowired
	ApplicationProperties appProps;

	public EPASChildren getById(String id) {
		EPASChildren epasChildren = rt.getForObject(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/child/show?id={id}", EPASChildren.class, id);
		log.info("Retrieved Child: {}", epasChildren);
		return epasChildren;
	}

	public List<EPASChildren> getByPersonId(String id) {
		ResponseEntity<List<EPASChildren>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/child/byPerson?id={id}", HttpMethod.GET, null,
				new ParameterizedTypeReference<List<EPASChildren>>() {
				}, id);
		List<EPASChildren> listEPASChildren = responseEntity.getBody();
		log.info("Retrieved Children: {}", listEPASChildren);
		return listEPASChildren;

	}

	public List<EPASChildren> getByPersonFiscalCode(String fc) {
		ResponseEntity<List<EPASChildren>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/child/byPerson?fiscalCode={fc}", HttpMethod.GET,
				null, new ParameterizedTypeReference<List<EPASChildren>>() {
				}, fc);
		List<EPASChildren> listEPASChildren = responseEntity.getBody();
		log.info("Retrieved Children: {}", listEPASChildren);
		return listEPASChildren;

	}

	public List<EPASChildren> getByPersonEmail(String email) {
		ResponseEntity<List<EPASChildren>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/child/byPerson?fiscalCode={email}", HttpMethod.GET,
				null, new ParameterizedTypeReference<List<EPASChildren>>() {
				}, email);
		List<EPASChildren> listEPASChildren = responseEntity.getBody();
		log.info("Retrieved Children: {}", listEPASChildren);
		return listEPASChildren;

	}

	public EPASChildren create(EPASChildrenDTO epasChildrenDTO) {
		ResponseEntity<EPASChildren> response = rt.postForEntity(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/child/create", epasChildrenDTO,
				EPASChildren.class);

		EPASChildren createdEPASChildren = response.getBody();

		log.info("Created Child: {}", createdEPASChildren);
		return createdEPASChildren;
	}

	public void updateById(String id, @Valid EPASChildrenDTO epasChildrenDTO) {
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("id", id);

		rt.put(appProps.getDatasourceEpasRest().getRestUrl() + "/v2/child/update?id={id}", epasChildrenDTO,
				uriVariables);

		log.info("Updated Child: {}", id);
		return;
	}

	public void deleteById(String id) {
		rt.delete(appProps.getDatasourceEpasRest().getRestUrl() + "/v2/child/delete?id={id}", id);
		log.info("Deleted Child with id: {}", id);
		return;
	}

}
