package it.cnr.isti.epasmed.epas.client;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import it.cnr.isti.epasmed.config.ApplicationProperties;
import it.cnr.isti.epasmed.epas.dto.EPASContractsDTO;
import it.cnr.isti.epasmed.epas.model.EPASContracts;

@Component
public class EPASContractsClient {

	private final Logger log = LoggerFactory.getLogger(getClass());

	@Autowired
	@Qualifier("RestTemplateForFirstUser")
	RestTemplate rt;

	@Autowired
	ApplicationProperties appProps;

	public EPASContracts getById(String id) {
		EPASContracts epasContracts = rt.getForObject(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/contracts/show/{id}", EPASContracts.class, id);
		log.info("Retrieved Contracts: {}", epasContracts);
		return epasContracts;
	}

	public List<EPASContracts> getByPersonId(String id) {
		ResponseEntity<List<EPASContracts>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/contracts/byPerson?id={id}", HttpMethod.GET, null,
				new ParameterizedTypeReference<List<EPASContracts>>() {
				}, id);
		List<EPASContracts> listEPASContracts = responseEntity.getBody();
		log.info("Retrieved Contracts: {}", listEPASContracts);
		return listEPASContracts;

	}

	public List<EPASContracts> getByPersonFiscalcode(String fc) {
		ResponseEntity<List<EPASContracts>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/contracts/byPerson?fiscalCode={fc}", HttpMethod.GET,
				null, new ParameterizedTypeReference<List<EPASContracts>>() {
				}, fc);
		List<EPASContracts> listEPASContracts = responseEntity.getBody();
		log.info("Retrieved Contracts: {}", listEPASContracts);
		return listEPASContracts;
	}

	public List<EPASContracts> getByPersonEmail(String email) {
		ResponseEntity<List<EPASContracts>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/contracts/byPerson?email={email}", HttpMethod.GET,
				null, new ParameterizedTypeReference<List<EPASContracts>>() {
				}, email);
		List<EPASContracts> listEPASContracts = responseEntity.getBody();
		log.info("Retrieved Contracts: {}", listEPASContracts);
		return listEPASContracts;
	}

	
	public EPASContracts create(EPASContractsDTO epasContractsDTO) {
		ResponseEntity<EPASContracts> response = rt.postForEntity(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/contracts/create", epasContractsDTO,
				EPASContracts.class);

		EPASContracts createdEPASContracts = response.getBody();

		log.info("Created Contracts: {}", createdEPASContracts);
		return createdEPASContracts;
	}

	public void updateById(String id, @Valid EPASContractsDTO epasContractsDTO) {
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("id", id);

		rt.put(appProps.getDatasourceEpasRest().getRestUrl() + "/v2/contracts/update?id={id}", epasContractsDTO,
				uriVariables);

		log.info("Updated Contracts id: {}", id);
		return;
	}

	// Continuazione di due contratti consecutivi
	// E' possibile impostare che un contratto è continuativo rispetto al precedente
	// e ne eredità le ferie non godute precedenti. Questa funzionalità è utilizzare
	// per esempio per alcune stabilizzazioni dove il dipendente si porta dietro dal
	// precedente contratto la situazione delle ferie non godute. Per impostare e
	// rimuovere che un contratto è continuativo rispetto al precedente è possibile
	// utilizzare i metodi
	/// rest/v2/contract/setPreviousContract
	/// rest/v2/contract/unsetPreviousContract
	// $ http -a isti_registry_manager DELETE
	// https://epas-demo.devel.iit.cnr.it/rest/v2/contract/setPreviousContract?id=4678
	// $ http -a isti_registry_manager DELETE
	// https://epas-demo.devel.iit.cnr.it/rest/v2/contract/unsetPreviousContract?id=4678

	public void setPreviousContract(String id) {
		rt.delete(appProps.getDatasourceEpasRest().getRestUrl() + "/v2/contracts/setPreviousContract?id={id}", id);
		log.info("Set Previous Contract for Contract id: {}", id);
		return;
	}

	public void setUnsetPreviousContract(String id) {
		rt.delete(appProps.getDatasourceEpasRest().getRestUrl() + "/v2/contracts/unsetPreviousContract?id={id}", id);
		log.info("Set Unset Previous Contract for Contract id: {}", id);
		return;
	}

	public void deleteById(String id) {
		rt.delete(appProps.getDatasourceEpasRest().getRestUrl() + "/v2/contracts/delete?id={id}", id);
		log.info("Deleted Contract id: {}", id);
		return;
	}

}
