package it.cnr.isti.epasmed.epas.client;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import it.cnr.isti.epasmed.config.ApplicationProperties;
import it.cnr.isti.epasmed.epas.dto.EPASGroupsDTO;
import it.cnr.isti.epasmed.epas.model.EPASGroups;

@Component
public class EPASGroupsClient {

	private final Logger log = LoggerFactory.getLogger(getClass());

	@Autowired
	@Qualifier("RestTemplateForFirstUser")
	RestTemplate rt;

	@Autowired
	ApplicationProperties appProps;

	public EPASGroups getById(String id) {
		EPASGroups epasGroups = rt.getForObject(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/groups/show?id={id}", EPASGroups.class, id);
		log.info("Retrieved Groups: {}", epasGroups);
		return epasGroups;
	}

	public List<EPASGroups> getList(String officeId) {
		ResponseEntity<List<EPASGroups>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/groups/list?id={officeId}", HttpMethod.GET, null,
				new ParameterizedTypeReference<List<EPASGroups>>() {
				}, officeId);
		List<EPASGroups> listEPASGroups = responseEntity.getBody();
		log.info("Retrieved Groups: {}", listEPASGroups);
		return listEPASGroups;
	}

	public EPASGroups create(EPASGroupsDTO epasGroupsDTO) {
		ResponseEntity<EPASGroups> response = rt.postForEntity(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/groups/create", epasGroupsDTO, EPASGroups.class);

		EPASGroups createdEPASGroups = response.getBody();

		log.info("Created Groups: {}", createdEPASGroups);
		return createdEPASGroups;
	}

	public void updateById(String id, @Valid EPASGroupsDTO epasGroupsDTO) {
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("id", id);

		rt.put(appProps.getDatasourceEpasRest().getRestUrl() + "/v2/groups/update?id={id}", epasGroupsDTO, uriVariables);

		log.info("Updated Groups id: {}", id);
		return;
	}

	public void deleteById(String id) {
		rt.delete(appProps.getDatasourceEpasRest().getRestUrl() + "/v2/groups/delete?id={id}", id);
		log.info("Deleted Group id: {}", id);
		return;
	}

	// $ http -a isti_registry_manager POST
	// https://epas-demo.devel.iit.cnr.it/rest/v2/groups/create
	// name="Gruppo Test" description="Gruppo ISTI di test"
	// officeId=201 managerId=4222 externalId="pimpaExternalId"

}
