package it.cnr.isti.epasmed.epas.client;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import it.cnr.isti.epasmed.config.ApplicationProperties;
import it.cnr.isti.epasmed.epas.model.EPASLeaves;

@Component
public class EPASLeavesClient {

	private final Logger log = LoggerFactory.getLogger(getClass());

	@Autowired
	@Qualifier("RestTemplateForSecondUser")
	RestTemplate rt;

	@Autowired
	ApplicationProperties appProps;

	public List<EPASLeaves> getByPersonId(String id, String year, String includeDetails) {
		log.info("Retrieving EPASLeaves by person id: {}", id);
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("id", id);
		uriVariables.put("year", year);
		uriVariables.put("includeDetails", includeDetails);

		ResponseEntity<List<EPASLeaves>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl()
						+ "/v2/leaves/byPersonAndYear?id={id}&year={year}&includeDetails={includeDetails}",
				HttpMethod.GET, null, new ParameterizedTypeReference<List<EPASLeaves>>() {
				}, uriVariables);
		List<EPASLeaves> listEPASLeaves = responseEntity.getBody();
		log.info("Retrieved Leaves: {}", listEPASLeaves);
		return listEPASLeaves;
	}

	public List<EPASLeaves> getByFiscalcode(String fc, String year, String includeDetails) {
		log.info("Retrieving EPASLeaves by person fiscalcode: {}", fc);
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("fc", fc);
		uriVariables.put("year", year);
		uriVariables.put("includeDetails", includeDetails);

		ResponseEntity<List<EPASLeaves>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl()
						+ "/v2/leaves/byPersonAndYear?fiscalCode={fc}&year={year}&includeDetails={includeDetails}",
				HttpMethod.GET, null, new ParameterizedTypeReference<List<EPASLeaves>>() {
				}, uriVariables);
		List<EPASLeaves> listEPASLeaves = responseEntity.getBody();
		log.info("Retrieved Leaves: {}", listEPASLeaves);
		return listEPASLeaves;

	}

	public List<EPASLeaves> getByPersonEmail(String email, String year, String includeDetails) {
		log.info("Retrieving EPASLeaves by person email: {}", email);
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("email", email);
		uriVariables.put("year", year);
		uriVariables.put("includeDetails", includeDetails);

		ResponseEntity<List<EPASLeaves>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl()
						+ "/v2/leaves/byPersonAndYear?email={email}&year={year}&includeDetails={includeDetails}",
				HttpMethod.GET, null, new ParameterizedTypeReference<List<EPASLeaves>>() {
				}, uriVariables);
		List<EPASLeaves> listEPASLeaves = responseEntity.getBody();
		log.info("Retrieved Leaves: {}", listEPASLeaves);
		return listEPASLeaves;
	}

	public List<EPASLeaves> getByOfficeId(String officeId, String year, String includeDetails) {
		log.info("Retrieving EPASLeaves by office codeId: {}", officeId);
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("id", officeId);
		uriVariables.put("year", year);
		uriVariables.put("includeDetails", includeDetails);

		ResponseEntity<List<EPASLeaves>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl()
						+ "/v2/leaves/byOfficeAndYear?id={id}&year={year}&includeDetails={includeDetails}",
				HttpMethod.GET, null, new ParameterizedTypeReference<List<EPASLeaves>>() {
				}, uriVariables);
		List<EPASLeaves> listEPASLeaves = responseEntity.getBody();
		log.info("Retrieved Leaves: {}", listEPASLeaves);
		return listEPASLeaves;
	}

}
