package it.cnr.isti.epasmed.epas.client;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import it.cnr.isti.epasmed.config.ApplicationProperties;
import it.cnr.isti.epasmed.epas.model.EPASOffSiteWorks;
import it.cnr.isti.epasmed.epas.model.EPASPersonDays;

@Component
public class EPASOffSiteWorksClient {

	private final Logger log = LoggerFactory.getLogger(getClass());

	@Autowired
	@Qualifier("RestTemplateForSecondUser")
	RestTemplate rt;

	@Autowired
	ApplicationProperties appProps;


	public List<EPASPersonDays> getListByPersonId(String id, String year, String month) {
		log.info("Retrieving Off Site Works List by id: {}", id);
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("id", id);
		uriVariables.put("year", year);
		uriVariables.put("month", month);

		ResponseEntity<List<EPASPersonDays>> responseEntity = rt.exchange(appProps.getDatasourceEpasRest().getRestUrl()
				+ "/v3/personDays/offSiteWorkByPersonAndMonth?id={id}&year={year}&month={month}",
				HttpMethod.GET, null, new ParameterizedTypeReference<List<EPASPersonDays>>() {
				}, uriVariables);
		List<EPASPersonDays> listEPASPersonDays = responseEntity.getBody();
		log.info("Retrieved Off Site Works List: {}", listEPASPersonDays);
		return listEPASPersonDays;
	}
	
	public List<EPASPersonDays> getListByPersonFiscalCode(String fc, String year, String month) {
		log.info("Retrieving Off Site Works List by fiscalcode: {}", fc);
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("fiscalCode", fc);
		uriVariables.put("year", year);
		uriVariables.put("month", month);

		ResponseEntity<List<EPASPersonDays>> responseEntity = rt.exchange(appProps.getDatasourceEpasRest().getRestUrl()
				+ "/v3/personDays/offSiteWorkByPersonAndMonth?fiscalCode={fiscalCode}&year={year}&month={month}",
				HttpMethod.GET, null, new ParameterizedTypeReference<List<EPASPersonDays>>() {
				}, uriVariables);
		List<EPASPersonDays> listEPASPersonDays = responseEntity.getBody();
		log.info("Retrieved Off Site Works List: {}", listEPASPersonDays);
		return listEPASPersonDays;
	}
	
	
	public List<EPASPersonDays> getListByPersonEmail(String email, String year, String month) {
		log.info("Retrieving Off Site Works List by email: {}", email);
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("email", email);
		uriVariables.put("year", year);
		uriVariables.put("month", month);

		ResponseEntity<List<EPASPersonDays>> responseEntity = rt.exchange(appProps.getDatasourceEpasRest().getRestUrl()
				+ "/v3/personDays/offSiteWorkByPersonAndMonth?email={email}&year={year}&month={month}",
				HttpMethod.GET, null, new ParameterizedTypeReference<List<EPASPersonDays>>() {
				}, uriVariables);
		List<EPASPersonDays> listEPASPersonDays = responseEntity.getBody();
		log.info("Retrieved Off Site Works List: {}", listEPASPersonDays);
		return listEPASPersonDays;
	}

	
	public List<EPASOffSiteWorks> getListByOfficeCodeId(String id, String year, String month) {
		log.info("Retrieving Off Site Works List by office code id: {}", id);
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("sedeId", id);
		uriVariables.put("year", year);
		uriVariables.put("month", month);

		ResponseEntity<List<EPASOffSiteWorks>> responseEntity = rt.exchange(appProps.getDatasourceEpasRest().getRestUrl()
				+ "/v3/personDays/offSiteWorkByOfficeAndMonth?sedeId={sedeId}&year={year}&month={month}",
				HttpMethod.GET, null, new ParameterizedTypeReference<List<EPASOffSiteWorks>>() {
				}, uriVariables);
		List<EPASOffSiteWorks> listEPASOffSiteWorks = responseEntity.getBody();
		log.info("Retrieved Off Site Works List: {}", listEPASOffSiteWorks);
		return listEPASOffSiteWorks;
	}

}
