package it.cnr.isti.epasmed.epas.client;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import it.cnr.isti.epasmed.config.ApplicationProperties;
import it.cnr.isti.epasmed.epas.dto.EPASPersonsDTO;
import it.cnr.isti.epasmed.epas.model.EPASPersons;

@Component
public class EPASPersonsClient {

	private final Logger log = LoggerFactory.getLogger(getClass());

	@Autowired
	@Qualifier("RestTemplateForFirstUser")
	RestTemplate rt;

	@Autowired
	ApplicationProperties appProps;

	public EPASPersons getById(String id) {
		EPASPersons epasPersons = rt.getForObject(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/persons/show/{id}", EPASPersons.class, id);
		log.info("Retrieved Persons: {}", epasPersons);
		return epasPersons;
	}

	public EPASPersons getByFiscalCode(String fc) {
		EPASPersons epasPersons = rt.getForObject(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/persons/show?fiscalCode={fc}", EPASPersons.class, fc);
		log.info("Retrieved Persons: {}", epasPersons);
		return epasPersons;
	}

	public List<EPASPersons> getList(String officeId) {
		ResponseEntity<List<EPASPersons>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/persons/list?id={officeId}", HttpMethod.GET, null,
				new ParameterizedTypeReference<List<EPASPersons>>() {
				}, officeId);
		List<EPASPersons> listEPASPersons = responseEntity.getBody();
		log.info("Retrieved Persons: {}", listEPASPersons);
		return listEPASPersons;
	}

	public EPASPersons create(EPASPersonsDTO epasPersonsDTO) {
		ResponseEntity<EPASPersons> response = rt.postForEntity(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/persons/create", epasPersonsDTO, EPASPersons.class);

		EPASPersons createdEPASPersons = response.getBody();

		log.info("Created Persons: {}", createdEPASPersons);
		return createdEPASPersons;
	}

	public void updateById(String id, @Valid EPASPersonsDTO epasPersonsDTO) {
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("id", id);

		rt.put(appProps.getDatasourceEpasRest().getRestUrl() + "/v2/persons/update?id={id}", epasPersonsDTO, uriVariables);

		log.info("Updated Persons: {}", id);
		return;
	}

	public void updateByFiscalCode(String fc, EPASPersonsDTO epasPersonsDTO) {
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("fiscalCode", fc);

		rt.put(appProps.getDatasourceEpasRest().getRestUrl() + "/v2/persons/update?fiscalCode={fiscalCode}",
				epasPersonsDTO, uriVariables);

		log.info("Updated Person: {}", fc);
		return;
	}

	public void deleteById(String id) {
		rt.delete(appProps.getDatasourceEpasRest().getRestUrl() + "/v2/persons/delete?id={id}", id);
		log.info("Deleted Person with id: {}", id);
		return;
	}

	public void deleteByFiscalCode(String fc) {
		rt.delete(appProps.getDatasourceEpasRest().getRestUrl() + "/v2/persons/delete?fiscalCode={fc}", fc);
		log.info("Delete Persons by fiscalcode: {}", fc);
		return;
	}

	/*
	 * HttpHeaders headers = new HttpHeaders();
	 * headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
	 * 
	 * MultiValueMap<String, String> map= new LinkedMultiValueMap<String, String>();
	 * map.add("name", epasPersons.getName()); map.add("surname",
	 * epasPersons.getSurname()); map.add("fiscalCode",
	 * epasPersons.getFiscalCode()); map.add("email", epasPersons.getEmail());
	 * map.add("qualification", epasPersons.getQualifcation()); map.add("officeId",
	 * epasPersons.getOffice().getId());
	 * 
	 * HttpEntity<MultiValueMap<String, String>> request = new
	 * HttpEntity<MultiValueMap<String, String>>(map, headers);
	 * 
	 * ResponseEntity<EPASPersons> response = rt.postForEntity(
	 * appProps.getDatasourceEpasRest().getRestUrl() + "/persons/create", request ,
	 * EPASPersons.class );
	 */
	// ResponseEntity<EPASPersons> response = rt.postForEntity(
	// appProps.getDatasourceEpasRest().getRestUrl()
	// + "/persons/create?name={name}&surname={surname}&email={email}"
	// +
	// "&qualification={qualification}&officeId={officeId}&fiscalCode={fiscalCode}",
	// null, EPASPersons.class, uriVariables);

}
