package it.cnr.isti.epasmed.epas.client;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import it.cnr.isti.epasmed.config.ApplicationProperties;
import it.cnr.isti.epasmed.epas.model.EPASTimeCards;

@Component
public class EPASTimeCardsClient {

	private final Logger log = LoggerFactory.getLogger(getClass());

	@Autowired
	@Qualifier("RestTemplateForSecondUser")
	RestTemplate rt;

	@Autowired
	ApplicationProperties appProps;

	public EPASTimeCards getByPersonId(String id, String year, String month) {
		log.info("Retrieving EPASTimeCard by person id: {}", id);
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("id", id);
		uriVariables.put("year", year);
		uriVariables.put("month", month);
		
		EPASTimeCards epasTimeCard = rt.getForObject(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v3/persondays/getMonthSituationByPerson?id={id}&year={year}&month={month}", 
				EPASTimeCards.class,uriVariables);
		log.info("Retrieved EPASTimeCard: {}", epasTimeCard);
		return epasTimeCard;
	}

	public EPASTimeCards getByPersonFiscalCode(String fc, String year, String month) {
		log.info("Retrieving EPASTimeCard by person fiscalcode: {}", fc);
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("fc", fc);
		uriVariables.put("year", year);
		uriVariables.put("month", month);
		
		EPASTimeCards epasTimeCard = rt.getForObject(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v3/persondays/getMonthSituationByPerson?fiscalCode={fc}&year={year}&month={month}", 
				EPASTimeCards.class,uriVariables);
		log.info("Retrieved EPASTimeCard: {}", epasTimeCard);
		return epasTimeCard;
	}


	public EPASTimeCards getByPersonEmail(String email, String year, String month) {
		log.info("Retrieving EPASTimeCard by person email: {}", email);
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("email", email);
		uriVariables.put("year", year);
		uriVariables.put("month", month);
		
		EPASTimeCards epasTimeCard = rt.getForObject(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v3/persondays/getMonthSituationByPerson?email={email}&year={year}&month={month}", 
				EPASTimeCards.class,uriVariables);
		log.info("Retrieved EPASTimeCard: {}", epasTimeCard);
		return epasTimeCard;
	}
	

	public List<EPASTimeCards> getByOfficeCodeId(String id, String year, String month) {
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("sedeId", id);
		uriVariables.put("year", year);
		uriVariables.put("month", month);
		
		ResponseEntity<List<EPASTimeCards>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v3/persondays/getmonthsituationbyoffice?sedeId={sedeId}&year={year}&month={month}", HttpMethod.GET, null,
				new ParameterizedTypeReference<List<EPASTimeCards>>() {
				}, uriVariables);
		List<EPASTimeCards> listEPASTimeCards = responseEntity.getBody();
		log.info("Retrieved EPASTimeCards: {}", listEPASTimeCards);
		return listEPASTimeCards;
	}


	

	
	
}
