package it.cnr.isti.epasmed.epas.client;

import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import it.cnr.isti.epasmed.config.ApplicationProperties;
import it.cnr.isti.epasmed.epas.model.EPASValidates;

@Component
public class EPASValidatesClient {

	private final Logger log = LoggerFactory.getLogger(getClass());

	@Autowired
	@Qualifier("RestTemplateForSecondUser")
	RestTemplate rt;

	@Autowired
	ApplicationProperties appProps;

	public String isByPersonEmail(String email, String year, String month) {
		log.info("Retrieving EPASValidates status by person email: {}", email);
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("email", email);
		uriVariables.put("year", year);
		uriVariables.put("month", month);
		
		String epasValidates = rt.getForObject(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/certifications/getMonthValidationStatusByPerson?email={email}&year={year}&month={month}", 
				String.class,uriVariables);
		log.info("Retrieved EPASValidated status: {}", epasValidates);
		return epasValidates;
	}


	public EPASValidates getByOfficeCodeId(String id, String year, String month) {
		log.info("Retrieving EPASValidates for office codeId: {}", id);
		
		Map<String, String> uriVariables = new HashMap<>();
		uriVariables.put("sedeId", id);
		uriVariables.put("year", year);
		uriVariables.put("month", month);
		
		EPASValidates epasValidates = rt.getForObject(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/certifications/getMonthValidationStatusByOffice?sedeId={sedeId}&year={year}&month={month}", 
				EPASValidates.class,uriVariables);
		log.info("Retrieved EPASValidates: {}", epasValidates);
		return epasValidates;
	}
}
