package it.cnr.isti.epasmed.epas.client;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import it.cnr.isti.epasmed.config.ApplicationProperties;
import it.cnr.isti.epasmed.epas.model.EPASWorkingTimeTypes;

@Component
public class EPASWorkingTimeTypesClient {

	private final Logger log = LoggerFactory.getLogger(getClass());

	@Autowired
	@Qualifier("RestTemplateForFirstUser")
	RestTemplate rt;

	@Autowired
	ApplicationProperties appProps;

	public EPASWorkingTimeTypes getById(String id) {
		EPASWorkingTimeTypes epasWorkingTimeTypes = rt.getForObject(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/workingtimetypes/show/{id}",
				EPASWorkingTimeTypes.class, id);
		log.info("Retrieved Working Time Type : {}", epasWorkingTimeTypes);
		return epasWorkingTimeTypes;
	}

	public List<EPASWorkingTimeTypes> getListByOfficeCodeId(String officeCodeId) {
		ResponseEntity<List<EPASWorkingTimeTypes>> responseEntity = rt.exchange(
				appProps.getDatasourceEpasRest().getRestUrl() + "/v2/workingtimetypes/list?codeId={officeCodeId}", HttpMethod.GET,
				null, new ParameterizedTypeReference<List<EPASWorkingTimeTypes>>() {
				}, officeCodeId);
		List<EPASWorkingTimeTypes> listEPASWorkingTimeTypes = responseEntity.getBody();
		log.info("Retrieved Working Time Types: {}", listEPASWorkingTimeTypes);
		return listEPASWorkingTimeTypes;
	}

}
