package it.cnr.isti.epasmed.epas.service;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.LinkedHashMap;
import java.util.List;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import it.cnr.isti.epasmed.epas.client.EPASContractsClient;
import it.cnr.isti.epasmed.epas.dto.EPASContractsDTO;
import it.cnr.isti.epasmed.epas.model.EPASContracts;
import it.cnr.isti.epasmed.epas.model.EPASPersons;

@Service
public class EPASContractsService {

	private static final Logger logger = LoggerFactory.getLogger(EPASContractsService.class);
	private final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

	@Autowired
	EPASContractsClient epasContractsClient;

	@Autowired
	EPASPersonsService epasPersonsService;

	public EPASContracts getById(String id) {
		return epasContractsClient.getById(id);
	}

	public List<EPASContracts> getByPersonId(String id) {
		return epasContractsClient.getByPersonId(id);
	}

	public List<EPASContracts> getByPersonFiscalcode(String fc) {
		return epasContractsClient.getByPersonFiscalcode(fc);
	}

	public List<EPASContracts> getByPersonEmail(String email) {
		return epasContractsClient.getByPersonEmail(email);
	}

	public EPASContracts create(EPASContractsDTO epasContractsDTO) {
		return epasContractsClient.create(epasContractsDTO);
	}

	public void updateById(String id, @Valid EPASContractsDTO epasContractsDTO) {
		epasContractsClient.updateById(id, epasContractsDTO);
	}

	public void setPreviousContract(String id) {
		epasContractsClient.setPreviousContract(id);
	}

	public void setUnsetPreviousContract(String id) {
		epasContractsClient.setUnsetPreviousContract(id);
	}

	public void deleteById(String id) {
		epasContractsClient.deleteById(id);
	}

	public LinkedHashMap<String, EPASContracts> getMap(String officeId) {
		List<EPASPersons> epasPersonsList = epasPersonsService.getList(officeId);
		LinkedHashMap<String, EPASContracts> lastContractsMap = new LinkedHashMap<>();

		for (EPASPersons epasPerson : epasPersonsList) {
			logger.debug("EPASPerson: {}", epasPerson);
			List<EPASContracts> listContracts = null;
			try {
				listContracts = getByPersonId(epasPerson.getId());
			} catch (Exception e) {
				logger.error("Error retrieving contracts list for person: {}", epasPerson.getId());
			}

			if (listContracts == null || listContracts.isEmpty()) {
				logger.error("There is no contract for this person Id: {}", epasPerson.getId());
				lastContractsMap.put(epasPerson.getId(), null);
				continue;
			}

			String lastContractId = null;
			LocalDate lastContractDate = null;
			LocalDate currentContractDate = null;
			for (EPASContracts contract : listContracts) {
				currentContractDate = LocalDate.parse(contract.getBeginDate(), formatter);
				if (lastContractDate == null || currentContractDate.isAfter(lastContractDate)) {
					lastContractDate = currentContractDate;
					lastContractId = contract.getId();
				}
			}

			if (lastContractId == null || lastContractId.isEmpty()) {
				logger.error("There is no last contract for this person id: {}", epasPerson.getId());
				lastContractsMap.put(epasPerson.getId(), null);
				continue;
			}
			logger.debug("Found Last Contract Id: {}", lastContractId);

			EPASContracts lastContract = getById(lastContractId);

			if (lastContract == null) {
				logger.error("There is no last contract by Id: {}", lastContractId);
				lastContractsMap.put(epasPerson.getId(), null);
				continue;
			}
			lastContractsMap.put(epasPerson.getId(), lastContract);
			
		}
		
		return lastContractsMap;
	}

}
