package it.cnr.isti.epasmed.service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import it.cnr.isti.epasmed.domain.Leaves;
import it.cnr.isti.epasmed.repository.LeavesRepository;

/**
 * Service class for managing Leaves.
 */
@Service
@Transactional("epasMedTransactionManager")
public class LeavesService {

	private final Logger log = LoggerFactory.getLogger(LeavesService.class);

	private final LeavesRepository leavesRepository;

	// @Autowired
	// FormattingConversionService conversionDateTimeService;

	public LeavesService(LeavesRepository leavesRepository) {
		this.leavesRepository = leavesRepository;
	}

	@Transactional(value = "epasMedTransactionManager", readOnly = true)
	public Page<Leaves> getAllLeaves(Pageable pageable) {
		log.debug("LeavesService  getAllLeaves(): {}", pageable);
		leavesRepository.findAll();
		return leavesRepository.findAll(pageable);
	}

	@Transactional(value = "epasMedTransactionManager", readOnly = true)
	public List<Leaves> getAllLeaves() {
		log.debug("LeavesService  getAllLeaves()");
		return leavesRepository.findAll();
	}

	@Transactional(value = "epasMedTransactionManager", readOnly = true)
	public Optional<Leaves> getLeavesById(Integer id) {
		return leavesRepository.findById(id);
	}

	public Leaves createLeaves(Leaves leavesDTO) {
		log.debug("Creating new Leaves: {}", leavesDTO);
		Leaves leaves = new Leaves();
		leaves.setId(leavesDTO.getId());
		leaves.setIdPersona(leavesDTO.getIdPersona());
		leaves.setCf(leavesDTO.getCf());
		leaves.setDataInizio(leavesDTO.getDataInizio());
		leaves.setDataFine(leavesDTO.getDataFine());
		leaves.setCodiceAssenzaCodice(leavesDTO.getCodiceAssenzaCodice());
		leaves.setCodiceAssenzaCnr(leavesDTO.getCodiceAssenzaCnr());
		leaves.setCodiceAssenzaDescrizione(leavesDTO.getCodiceAssenzaDescrizione());
		leaves.setDurata(leavesDTO.getDurata());
		leaves.setDataMod(leavesDTO.getDataMod());
		leavesRepository.save(leaves);

		log.debug("Created Leaves: {}", leaves);
		return leaves;
	}

	/**
	 * Update all information for a specific leaves, and return the modified leaves.
	 *
	 * @param leavesDTO leave to update.
	 * @return updated table.
	 */
	public Optional<Leaves> updateLeaves(Leaves leavesDTO) {
		Optional<Leaves> l = leavesRepository.findById(leavesDTO.getId());
		if (l.isPresent()) {
			Leaves leaves = l.get();
			leaves.setId(leavesDTO.getId());
			leaves.setIdPersona(leavesDTO.getIdPersona());
			leaves.setCf(leavesDTO.getCf());
			leaves.setDataInizio(leavesDTO.getDataInizio());
			leaves.setDataFine(leavesDTO.getDataInizio());
			leaves.setCodiceAssenzaCodice(leavesDTO.getCodiceAssenzaCodice());
			leaves.setCodiceAssenzaCnr(leavesDTO.getCodiceAssenzaCnr());
			leaves.setCodiceAssenzaDescrizione(leavesDTO.getCodiceAssenzaDescrizione());
			leaves.setDurata(leavesDTO.getDurata());
			leaves.setDataMod(leavesDTO.getDataMod());
			leaves = leavesRepository.save(leaves);
			log.debug("Changed Information for Leaves: {}", leaves);
			return Optional.of(leaves);
		} else {
			return l;
		}
	}

	public void deleteLeaves(Integer id) {
		leavesRepository.findById(id).ifPresent(leaves -> {
			leavesRepository.delete(leaves);
			log.debug("Deleted Leaves: {}", leaves);
		});
	}

	
	public void updateLeaves(String id, String idPersona, String cf, String dataInizio, String dataFine,
			String codiceAssenzaCodice, String codiceAssenzaCnr, String codiceAssenzaDescrizione, String durata,
			String dataMod) {

		Optional.of(leavesRepository.findById(Integer.valueOf(id))).filter(Optional::isPresent).map(Optional::get)
				.map(leaves -> {
					LocalDate dataI = null;
					LocalDate dataF = null;
					LocalDateTime dataM = null;

					if (dataInizio != null && !dataInizio.isEmpty()) {
						dataI = LocalDate.parse(dataInizio);
					}
					if (dataFine != null && !dataFine.isEmpty()) {
						dataF = LocalDate.parse(dataFine);
					}

					if (dataMod != null && !dataMod.isEmpty()) {
						dataM = LocalDateTime.parse(dataMod);
					}

					leaves.setIdPersona(Integer.valueOf(idPersona));
					leaves.setCf(cf);
					leaves.setDataInizio(dataI);
					leaves.setDataFine(dataF);
					leaves.setCodiceAssenzaCodice(Integer.parseInt(codiceAssenzaCodice));
					leaves.setCodiceAssenzaCnr(codiceAssenzaCnr);
					leaves.setCodiceAssenzaDescrizione(codiceAssenzaDescrizione);
					leaves.setDurata(Integer.parseInt(durata));
					leaves.setDataMod(dataM);
					log.debug("Changed Information for Leaves: {}", leaves);
					return leaves;
				});
	}

}
