package it.cnr.isti.epasmed.service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import it.cnr.isti.epasmed.domain.PersOrario;
import it.cnr.isti.epasmed.repository.PersOrarioRepository;

/**
 * Service class for managing PersOrario.
 */
@Service
@Transactional("epasMedTransactionManager")
public class PersOrarioService {

	private final Logger log = LoggerFactory.getLogger(PersOrarioService.class);

	private final PersOrarioRepository persOrarioRepository;

	public PersOrarioService(PersOrarioRepository persOrarioRepository) {
		this.persOrarioRepository = persOrarioRepository;
	}

	@Transactional(value = "epasMedTransactionManager", readOnly = true)
	public Page<PersOrario> getAllPersOrario(Pageable pageable) {
		log.debug("PersOrarioService  getAllPersOrario(): {}", pageable);
		persOrarioRepository.findAll();
		return persOrarioRepository.findAll(pageable);
	}

	@Transactional(value = "epasMedTransactionManager", readOnly = true)
	public List<PersOrario> getAllPersOrario() {
		log.debug("PersOrarioService  getAllPersOrario()");
		return persOrarioRepository.findAll();
	}

	@Transactional(value = "epasMedTransactionManager", readOnly = true)
	public Optional<PersOrario> getPersOrarioById(Integer id) {
		return persOrarioRepository.findById(id);
	}

	public PersOrario createPersOrario(PersOrario persOrarioDTO) {
		log.debug("Creating new PersOrario: {}", persOrarioDTO);
		PersOrario persOrario = new PersOrario();
		persOrario.setId(persOrarioDTO.getId());
		persOrario.setIdPersona(persOrarioDTO.getIdPersona());
		persOrario.setCf(persOrarioDTO.getCf());
		persOrario.setDal(persOrarioDTO.getDal());
		persOrario.setAl(persOrarioDTO.getAl());
		persOrario.setDescrizione(persOrarioDTO.getDescrizione());
		persOrario.setLun(persOrarioDTO.getLun());
		persOrario.setMar(persOrarioDTO.getMar());
		persOrario.setMer(persOrarioDTO.getMer());
		persOrario.setGio(persOrarioDTO.getGio());
		persOrario.setVen(persOrarioDTO.getVen());
		persOrario.setSab(persOrarioDTO.getSab());
		persOrario.setPercentuale(persOrarioDTO.getPercentuale());
		persOrario.setTurno(persOrarioDTO.getTurno());
		persOrario.setOreTurno(persOrarioDTO.getOreTurno());
		persOrario.setFestivo(persOrarioDTO.getFestivo());
		persOrario.setNotturno(persOrarioDTO.getNotturno());
		persOrario.setDataMod(persOrarioDTO.getDataMod());
		persOrarioRepository.save(persOrario);

		log.debug("Created PersOrario: {}", persOrario);
		return persOrario;
	}

	/**
	 * Update all information
	 *
	 * @param persOrarioDTO pers orario to update.
	 * @return updated table.
	 */
	public Optional<PersOrario> updatePersOrario(PersOrario persOrarioDTO) {
		Optional<PersOrario> po = persOrarioRepository.findById(persOrarioDTO.getId());
		if (po.isPresent()) {
			PersOrario persOrario = po.get();
			persOrario.setId(persOrarioDTO.getId());
			persOrario.setIdPersona(persOrarioDTO.getIdPersona());
			persOrario.setCf(persOrarioDTO.getCf());
			persOrario.setDal(persOrarioDTO.getDal());
			persOrario.setAl(persOrarioDTO.getAl());
			persOrario.setDescrizione(persOrarioDTO.getDescrizione());
			persOrario.setLun(persOrarioDTO.getLun());
			persOrario.setMar(persOrarioDTO.getMar());
			persOrario.setMer(persOrarioDTO.getMer());
			persOrario.setGio(persOrarioDTO.getGio());
			persOrario.setVen(persOrarioDTO.getVen());
			persOrario.setSab(persOrarioDTO.getSab());
			persOrario.setPercentuale(persOrarioDTO.getPercentuale());
			persOrario.setTurno(persOrarioDTO.getTurno());
			persOrario.setOreTurno(persOrarioDTO.getOreTurno());
			persOrario.setFestivo(persOrarioDTO.getFestivo());
			persOrario.setNotturno(persOrarioDTO.getNotturno());
			persOrario.setDataMod(persOrarioDTO.getDataMod());
			persOrario = persOrarioRepository.save(persOrario);
			log.debug("Changed Information for : {}", persOrario);
			return Optional.of(persOrario);
		} else {
			return po;
		}
	}

	public void deletePersOrario(Integer id) {
		persOrarioRepository.findById(id).ifPresent(perOrario -> {
			persOrarioRepository.delete(perOrario);
			log.debug("Deleted PersOrario: {}", perOrario);
		});
	}

	public void updatePersOrario(String id, String idPersona, String cf, String dal, String al, String descrizione,
			String lun, String mar, String mer, String gio, String ven, String sab, String percentuale, String turno,
			String oreTurno, String festivo, String notturno, String dataMod) {

		Optional.of(persOrarioRepository.findById(Integer.valueOf(id))).filter(Optional::isPresent).map(Optional::get)
				.map(persOrario -> {
					LocalDate dataI = null;
					LocalDate dataF = null;
					LocalDateTime dataM = null;

					if (dal != null && !dal.isEmpty()) {
						dataI = LocalDate.parse(dal);
					}
					if (al != null && !al.isEmpty()) {
						dataF = LocalDate.parse(al);
					}

					if (dataMod != null && !dataMod.isEmpty()) {
						dataM = LocalDateTime.parse(dataMod);
					}

					Integer lu = null;
					try {
						lu = Integer.valueOf(lun);
					} catch (NumberFormatException e) {
						log.error(e.getLocalizedMessage(), e);
					}
					Integer ma = null;
					try {
						ma = Integer.valueOf(mar);
					} catch (NumberFormatException e) {
						log.error(e.getLocalizedMessage(), e);
					}
					Integer me = null;
					try {
						me = Integer.valueOf(mer);
					} catch (NumberFormatException e) {
						log.error(e.getLocalizedMessage(), e);
					}
					Integer gi = null;
					try {
						gi = Integer.valueOf(gio);
					} catch (NumberFormatException e) {
						log.error(e.getLocalizedMessage(), e);
					}
					Integer ve = null;
					try {
						ve = Integer.valueOf(ven);
					} catch (NumberFormatException e) {
						log.error(e.getLocalizedMessage(), e);
					}
					Integer sa = null;
					try {
						sa = Integer.valueOf(sab);
					} catch (NumberFormatException e) {
						log.error(e.getLocalizedMessage(), e);
					}

					Integer perc = null;
					try {
						perc = Integer.valueOf(percentuale);
					} catch (NumberFormatException e) {
						log.error(e.getLocalizedMessage(), e);
					}

					Integer oreTu = null;
					try {
						oreTu = Integer.valueOf(oreTurno);
					} catch (NumberFormatException e) {
						log.error(e.getLocalizedMessage(), e);
					}

					persOrario.setIdPersona(Integer.valueOf(idPersona));
					persOrario.setCf(cf);
					persOrario.setDal(dataI);
					persOrario.setAl(dataF);
					persOrario.setDescrizione(descrizione);
					persOrario.setLun(lu);
					persOrario.setMar(ma);
					persOrario.setMer(me);
					persOrario.setGio(gi);
					persOrario.setVen(ve);
					persOrario.setSab(sa);
					persOrario.setPercentuale(perc);
					persOrario.setTurno(turno);
					persOrario.setOreTurno(oreTu);
					persOrario.setFestivo(festivo);
					persOrario.setNotturno(notturno);
					persOrario.setDataMod(dataM);
					log.debug("Changed Information for PersOrario: {}", persOrario);
					return persOrario;
				});
	}

}
