package it.cnr.isti.epasmed.service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.convert.ConversionService;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import it.cnr.isti.epasmed.domain.TabsSI;
import it.cnr.isti.epasmed.repository.TabsSIRepository;

/**
 * Service class for managing users.
 */
@Service
@Transactional("epasMedTransactionManager")
public class TabsSIService {

	private final Logger log = LoggerFactory.getLogger(TabsSIService.class);

	private final TabsSIRepository tabsSIRepository;

	@Autowired
	ConversionService conversionService;

	// @Autowired
	// FormattingConversionService conversionDateTimeService;

	public TabsSIService(TabsSIRepository tabsSIRepository) {
		this.tabsSIRepository = tabsSIRepository;
	}

	@Transactional(value = "epasMedTransactionManager", readOnly = true)
	public Page<TabsSI> getAllTabsSI(Pageable pageable) {
		log.debug("TabsSIService  getAllTabsSI(): {}", pageable);
		tabsSIRepository.findAll();
		return tabsSIRepository.findAll(pageable);
	}

	@Transactional(value = "epasMedTransactionManager", readOnly = true)
	public List<TabsSI> getAllTabsSI() {
		log.debug("TabsSIService  getAllTabsSI()");
		return tabsSIRepository.findAll();
	}

	@Transactional(value = "epasMedTransactionManager", readOnly = true)
	public Optional<TabsSI> getTabsSIById(Long id) {
		return tabsSIRepository.findById(id);
	}

	public TabsSI createTabsSI(TabsSI tabsSIDTO) {
		log.debug("Creating new TabsSI: {}", tabsSIDTO);
		TabsSI tabsSI = new TabsSI();
		tabsSI.setNome(tabsSIDTO.getNome());
		tabsSI.setOperazioni(tabsSIDTO.getOperazioni());
		tabsSI.setIdFlusso(tabsSIDTO.getIdFlusso());
		tabsSI.setLastUpdate(tabsSIDTO.getLastUpdate());
		tabsSIRepository.save(tabsSI);

		log.debug("Created TabsSI: {}", tabsSI);
		return tabsSI;
	}

	/**
	 * Update all information for a specific table, and return the modified table.
	 *
	 * @param tabsSI table to update.
	 * @return updated table.
	 */
	public Optional<TabsSI> updateTabsSI(TabsSI tabsSIDTO) {
		Optional<TabsSI> tSI = tabsSIRepository.findById(tabsSIDTO.getId());
		if (tSI.isPresent()) {
			TabsSI tabsSI = tSI.get();
			tabsSI.setNome(tabsSIDTO.getNome());
			tabsSI.setOperazioni(tabsSIDTO.getOperazioni());
			tabsSI.setIdFlusso(tabsSIDTO.getIdFlusso());
			tabsSI.setLastUpdate(tabsSIDTO.getLastUpdate());
			tabsSI = tabsSIRepository.save(tabsSI);
			log.debug("Changed Information for TabsSI: {}", tabsSI);
			return Optional.of(tabsSI);
		} else {
			return tSI;
		}
	}

	public void deleteTabsSI(Long id) {
		tabsSIRepository.findById(id).ifPresent(tabsSI -> {
			tabsSIRepository.delete(tabsSI);
			log.debug("Deleted TabsSI: {}", tabsSI);
		});
	}

	/**
	 * Update TabsSI information (id, nome, operazioni, id flusso, last update).
	 * 
	 * @param id         Identify
	 * @param nome       Nome table
	 * @param operazioni Operazioni
	 * @param idFlusso   Id Flusso
	 * @param lastUpdate Last Update
	 */
	public void updateTabSI(String id, String nome, String operazioni, String idFlusso, String lastUpdate) {
		LocalDateTime lastU = conversionService.convert(lastUpdate, LocalDateTime.class);
		// LocalDateTime lastU=conversionDateTimeService.convert(lastUpdate,
		// LocalDateTime.class);

		Optional.of(tabsSIRepository.findById(Long.valueOf(id))).filter(Optional::isPresent).map(Optional::get)
				.map(tabsSI -> {
					tabsSI.setNome(nome);
					tabsSI.setOperazioni(operazioni);
					tabsSI.setIdFlusso(Long.valueOf(idFlusso));
					tabsSI.setLastUpdate(lastU);
					log.debug("Changed Information for TabsSI: {}", tabsSI);
					return tabsSI;
				});
	}

}
