package it.cnr.isti.epasmed.sistemainformativo.repository;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.sql.DataSource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import it.cnr.isti.epasmed.sistemainformativo.model.SIAnagrafico;

@Repository
public class SIAnagraficoRepository {

	private static final Logger logger = LoggerFactory.getLogger(SIAnagraficoRepository.class);

	private JdbcTemplate jdbcTemplate;

	//
	public SIAnagraficoRepository(final @Qualifier("sistemaInformativoDataSource") DataSource dataSource) {
		super();
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	public List<SIAnagrafico> readNewFlux(Long flusso) {
		String query="select codicefiscale, cognome, nome, datanascita,"
				+ " numbadge, data_mod, flag_del, id_flusso, id "
				+ " from anagrafico "
				+ " where id_flusso > "
				+ flusso
				+ " and ( "
				+ " exists( "
				+ " select 1 "
				+ " from posizioni p "
				+ " where p.cf::text=codicefiscale and p.tipocontratto='Dipendente') "
				+ " or "
				+ " exists( "
				+ " select 1 "
				+ " from posizioni ps "
				+ " where ps.cf::text=codicefiscale and ps.profilo='Volontario Servizio Civile') "
				+ " )" + " order by codicefiscale, id_flusso, data_mod";
		
		List<SIAnagrafico> anagrafica= jdbcTemplate.query(query, new RowMapper<SIAnagrafico>() {
			@Override
			public SIAnagrafico mapRow(ResultSet rs, int rowNum) throws SQLException {
				SIAnagrafico a=new SIAnagrafico();
				a.setCodicefiscale(rs.getString(1));
				a.setCognome(rs.getString(2));
				a.setNome(rs.getString(3));
				a.setDatanascita(rs.getDate(4));
				a.setNumbadge(rs.getInt(5));
				a.setData_mod(rs.getTimestamp(6));
				a.setFlag_del(rs.getString(7));
				a.setId_flusso(rs.getLong(8));
				a.setId(rs.getInt(9));
				return a;
			}});	
		logger.debug("Anagrafica: {}", anagrafica);
		return anagrafica;
				
	}

	

}
