package it.cnr.isti.epasmed.sistemainformativo.repository;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.sql.DataSource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

@Repository
public class SIMasterLogRepository {

	private static final Logger logger = LoggerFactory.getLogger(SIMasterLogRepository.class);

	private JdbcTemplate jdbcTemplate;

	//
	public SIMasterLogRepository(final @Qualifier("sistemaInformativoDataSource") DataSource dataSource) {
		super();
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	public Long count() {
		logger.debug("Call MasterLogRepository Count()");
		return jdbcTemplate.queryForObject("select count(*) from master_log", Long.class);
	}

	public void startFluxReads() {
		logger.info("Iniziato Flusso in lettura");
		jdbcTemplate.update("INSERT INTO master_log (codice_flusso, tabelle, operazione, data_inizio_oper) "
				+ "VALUES ('epas','anagrafico,mail,telefoni,gruppi,gruppo_pers','L',current_timestamp)");
		logger.info("Flusso in lettura aperto");

	}

	public void closeFluxReads(String tabelleLette) throws Exception {
		logger.info("Chiusura Flusso in lettura");
		logger.info("Tabelle Lette: {}", tabelleLette);
		if (tabelleLette == null) {
			tabelleLette = " ";
		} else {
			if (tabelleLette.isEmpty()) {
				tabelleLette = " ";
			}
		}

		String query = "SELECT id_flusso FROM master_log"
				+ " WHERE codice_flusso='epas' AND operazione='L' AND data_fine_oper IS NULL"
				+ " ORDER BY id_flusso DESC LIMIT 1";

		List<Long> idsFlux = jdbcTemplate.query(query, new RowMapper<Long>() {
			@Override
			public Long mapRow(ResultSet rs, int rowNum) throws SQLException {
				Long idFlusso = rs.getLong("id_flusso");
				return idFlusso;
			}
		});
		logger.info("Flusso da chiudere: {}", idsFlux);

		if (idsFlux != null && !idsFlux.isEmpty()) {

			jdbcTemplate.update("UPDATE master_log SET data_fine_oper=current_timestamp, tabelle='" + tabelleLette
					+ "', esito_oper='OK' WHERE codice_flusso='epas' AND id_flusso=" + idsFlux.get(0) + " ");

			logger.info("Flusso in lettura chiuso");
		} else {
			logger.error("Flusso da chiudere non trovato: {}", idsFlux);
			throw new Exception("Errore chiudendo il flusso di lettura, id: " + idsFlux);

		}
	}

	public Long startFluxWrites() throws Exception {
		logger.info("Iniziato Flusso in scrittura");
		jdbcTemplate.update("INSERT INTO master_log (codice_flusso, tabelle, operazione, data_inizio_oper)"
				+ " VALUES ('epas','epas_cartellini, epas_cartellini_rendicontazioni," + " epas_orario,"
				+ " epas_lavoro_fuori_sede, epas_aspettative, epas_pers_orario'," + " 'S',current_timestamp)");
		logger.info("Flusso in scrittura aperto");

		String query = "SELECT id_flusso FROM master_log" + " WHERE codice_flusso='epas' AND operazione='S'"
				+ " AND data_fine_oper IS NULL" + " ORDER BY id_flusso DESC";

		List<Long> fluxIds = jdbcTemplate.query(query, new RowMapper<Long>() {
			@Override
			public Long mapRow(ResultSet rs, int rowNum) throws SQLException {
				Long idFlusso = rs.getLong("id_flusso");
				return idFlusso;
			}
		});
		Long fluxId = 0l;
		if (fluxIds != null && !fluxIds.isEmpty()) {
			fluxId = fluxIds.get(0);
			logger.info("Flusso in scrittura id: {}", fluxId);
		} else {
			logger.error("Errore inizializando il flusso di scrittura, id: {}", fluxIds);
			throw new Exception("Errore inizializando il flusso di scrittura, id: " + fluxIds);
		}

		return fluxId;
	}

	public void closeFluxWrites(Long fluxId, String tabelleScritte) {
		logger.info("Chiusura Flusso in scrittura");
		logger.info("Tabelle Scritte: {}", tabelleScritte);
		if (tabelleScritte == null) {
			tabelleScritte = " ";
		} else {
			if (tabelleScritte.isEmpty()) {
				tabelleScritte = " ";
			}
		}

		if (fluxId != null && fluxId.longValue() != 0) {

			jdbcTemplate.update("UPDATE master_log SET data_fine_oper=current_timestamp, tabelle='" + tabelleScritte
					+ "', esito_oper='OK' WHERE codice_flusso='epas' AND id_flusso=" + fluxId + " ");

			logger.info("Flusso in scrittura chiuso");
		} else {
			logger.info("Flusso in scrittura da chiudere non trovato: {}", fluxId);
		}
	}

}
