package it.cnr.isti.epasmed.sistemainformativo.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import it.cnr.isti.epasmed.sistemainformativo.repository.SIMasterLogRepository;


/**
 * Service class for managing MasterLog.
 */
@Service
@Transactional("sistemaInformativoTransactionManager")
public class SIMasterLogService {

	private static final Logger logger = LoggerFactory.getLogger(SIMasterLogService.class);

	private final SIMasterLogRepository masterLogRepository;

	public SIMasterLogService(SIMasterLogRepository masterLogRepository) {
		super();
		this.masterLogRepository = masterLogRepository;
	}
	
	
	public Long count() {
		logger.debug("Request MasterLog count()");
		return masterLogRepository.count();
	}
	
	public void startFluxReads() {
		logger.debug("Request Start Flux Reads");
		masterLogRepository.startFluxReads();
	}
	
	public void closeFluxReads(String tabelleLette) throws Exception {
		logger.debug("Request Close Flux Reads");
		masterLogRepository.closeFluxReads(tabelleLette);
	}
	
	public Long startFluxWrites() throws Exception {
		logger.debug("Request Start Flux Writes");
		return masterLogRepository.startFluxWrites();
	}
	
	public void closeFluxWrites(Long fluxId,String tabelleScritte) {
		logger.debug("Request Close Flux Writes");
		masterLogRepository.closeFluxWrites(fluxId,tabelleScritte);
	}
}