package it.cnr.isti.epasmed.web.rest.epas;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.epas.model.EPASAbsenceTypes;
import it.cnr.isti.epasmed.epas.service.EPASAbsenceTypesService;

@RestController
@RequestMapping("/api/epas")
public class EPASAbsenceTypesResource {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final EPASAbsenceTypesService epasAbsenceTypesService;

	public EPASAbsenceTypesResource(EPASAbsenceTypesService epasAbsenceTypesService) {
		this.epasAbsenceTypesService = epasAbsenceTypesService;

	}

	/**
	 * {@code GET /absenceTypes} : get the absence types list.
	 *
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the list of EPAS Absence Types, or with status
	 *         {@code 404 (Not Found)}.
	 */
	@GetMapping("/absenceTypes")
	public ResponseEntity<List<EPASAbsenceTypes>> getEPASAbsencesTypes() {
		logger.debug("REST request GET EPAS Absences Types");
		List<EPASAbsenceTypes> epasAbsenceTypesList = null;

		epasAbsenceTypesList = epasAbsenceTypesService.getAbsenceTypes();

		return ResponseUtil.wrapOrNotFound(Optional.of(epasAbsenceTypesList));
	}

	/**
	 * {@code GET /absenceTypesMap} : get the absence types map.
	 *
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the map of EPAS Absence Types Code and EPAS Absence Types
	 *         Description, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/absenceTypesMap")
	public ResponseEntity<LinkedHashMap<String, String>> getEPASAbsencesTypesMap() {
		logger.debug("REST request GET EPAS Absences Types Map");
		LinkedHashMap<String, String> epasAbsenceTypesMap = null;

		epasAbsenceTypesMap = epasAbsenceTypesService.getAbsenceTypesMap();

		return ResponseUtil.wrapOrNotFound(Optional.of(epasAbsenceTypesMap));
	}

}
