package it.cnr.isti.epasmed.web.rest.epas;

import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.epas.model.EPASCertifications;
import it.cnr.isti.epasmed.epas.service.EPASCertificationsService;

@RestController
@RequestMapping("/api/epas")
public class EPASCertificationsResource {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final EPASCertificationsService epasCertificationsService;

	public EPASCertificationsResource(EPASCertificationsService epasCertificationsService) {
		this.epasCertificationsService = epasCertificationsService;

	}

	/**
	 * {@code GET /certifications/getMonthSituation} : get certifications by person, year and month
	 *
	 * @param id the id of the person.
	 * @param fiscalCode the fiscal code of the person.
	 * @param email the email of the person.
	 * @param year the year.
	 * @param month the month.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Leaves, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/certifications/getMonthSituation")
	public ResponseEntity<EPASCertifications> getEPASCertificationByPerson(@RequestParam("id") Optional<String> id,
			@RequestParam("fiscalCode") Optional<String> fiscalCode,
			@RequestParam("email") Optional<String> email,
			@RequestParam("year") String year,@RequestParam("month") String month) {
		
		
		EPASCertifications epasCertifications = null;
		if (id.isPresent()) {
			logger.info("REST request to get ePAS Certification by Person: {}", id.get());
			epasCertifications = epasCertificationsService.getCertificationsByPersonId(id.get(), year, month);
		} else {
			if (fiscalCode.isPresent()) {
				logger.info("REST request to get ePAS Certification by Person fiscal code: {}", fiscalCode.get());
				epasCertifications = epasCertificationsService.getCertificationsByPersonFiscalCode(fiscalCode.get(), year, month);
			} else {
				if (email.isPresent()) {
					logger.info("REST request to get ePAS Certification by Person email: {}", email.get());
					epasCertifications = epasCertificationsService.getCertificationsByPersonEmail(email.get(), year, month);
				} else {
					return ResponseUtil.wrapOrNotFound(Optional.of(epasCertifications), HeaderUtil.createFailureAlert(applicationName,false,
							"","","Invalid parameter in call"));
				
				}
			}
		}
		logger.info("Retrieved Certification: {}", epasCertifications);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasCertifications));
	}

	
	/**
	 * {@code GET /certifications/getMonthSituationByOffice} : get certifications by office, year and month.
	 *
	 * @param officeCodeId the office codeId.
	 * @param year the year.
	 * @param month the month.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the list of EPAS Leaves, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/certifications/getMonthSituationByOffice")
	public ResponseEntity<List<EPASCertifications>> getEPASCertificationByOfficeCodeId(@RequestParam("officeCodeId") String officeCodeId,
			@RequestParam("year") String year, @RequestParam("month") String month) {
		logger.info("REST request to get ePAS Certification by office: codeId={}, year={}, month={}", officeCodeId, year, month);
		List<EPASCertifications> epasCertificationsList = epasCertificationsService.getCertificationsByOfficeCodeId(officeCodeId, year, month);
		logger.info("Retrieved Certification: {}", epasCertificationsList);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasCertificationsList));

	}


}
