package it.cnr.isti.epasmed.web.rest.epas;

import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.epas.model.EPASOffSiteWorks;
import it.cnr.isti.epasmed.epas.model.EPASPersonDays;
import it.cnr.isti.epasmed.epas.service.EPASOffSiteWorksService;

@RestController
@RequestMapping("/api/epas")
public class EPASOffSiteWorksResource {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final EPASOffSiteWorksService epasOffSiteWorksService;

	public EPASOffSiteWorksResource(EPASOffSiteWorksService epasOffSiteWorksService) {
		this.epasOffSiteWorksService = epasOffSiteWorksService;

	}
	
	/**
	 * {@code GET /offsiteworks/byPerson} : get off site works by person, year and month
	 *
	 * @param id the id of the person.
	 * @param fiscalCode the fiscal code of the person.
	 * @param email the email of the person.
	 * @param year the year.
	 * @param month the month.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Off Site Works List, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/offsiteworks/byPerson")
	public ResponseEntity<List<EPASPersonDays>> getListOffSiteWorksByPerson(@RequestParam("id") Optional<String> id,
			@RequestParam("fiscalCode") Optional<String> fiscalCode,
			@RequestParam("email") Optional<String> email,
			@RequestParam("year") String year,@RequestParam("month") String month) {
		
		
		List<EPASPersonDays> epasOffSiteWorksList = null;
		if (id.isPresent()) {
			logger.info("REST request to get ePAS Off Site Works list by Person id: {}", id.get());
			epasOffSiteWorksList = epasOffSiteWorksService.getListByPersonId(id.get(), year, month);
		} else {
			if (fiscalCode.isPresent()) {
				logger.info("REST request to get ePAS Off Site Works list by Person fiscalcode: {}", fiscalCode.get());
				epasOffSiteWorksList = epasOffSiteWorksService.getListByPersonFiscalCode(fiscalCode.get(), year, month);
			} else {
				if (email.isPresent()) {
					logger.info("REST request to get ePAS Off Site Works list by Person email: {}", email.get());
					epasOffSiteWorksList = epasOffSiteWorksService.getListByPersonEmail(email.get(), year, month);
				} else {
					return ResponseUtil.wrapOrNotFound(Optional.of(epasOffSiteWorksList), HeaderUtil.createFailureAlert(applicationName,false,
							"","","Invalid parameter in call"));
				
				}
			}
		}
		logger.info("Retrieved Off Site Works list: {}", epasOffSiteWorksList);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasOffSiteWorksList));
	}

	
	
	
	/**
	 * {@code GET /offsiteworks/byOffice} : get the time cards list.
	 *
	 * @param officeCodeId the id of the office.
	 * @param year the year
	 * @param month the month
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Off Site Works List, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/offsiteworks/byOffice")
	public ResponseEntity<List<EPASOffSiteWorks>> getOffSiteWorksByOffice(@RequestParam("codeId") String id,
			@RequestParam("year") String year, @RequestParam("month") String month) {
		logger.info("REST request to get ePAS Off Site Works list: officeCodeId={}, year={}, month={}", id,year,month);
		List<EPASOffSiteWorks> epasOffSiteWorksList = epasOffSiteWorksService.getListByOfficeCodeId(id, year, month);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasOffSiteWorksList));

	}
	
}
