package it.cnr.isti.epasmed.web.rest.epas;

import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.epas.dto.EPASPersonWorkingTimeDTO;
import it.cnr.isti.epasmed.epas.service.EPASPersonWorkingTimeService;

@RestController
@RequestMapping("/api/epas")
public class EPASPersonWorkingTimeResource {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final EPASPersonWorkingTimeService epasPersonWorkingTimeService;

	public EPASPersonWorkingTimeResource(EPASPersonWorkingTimeService epasWorkingTimeByPersonService) {
		this.epasPersonWorkingTimeService = epasWorkingTimeByPersonService;

	}

	/**
	 * {@code GET /personworkingtime/byPerson} : get personworkingtime by person
	 *
	 * @param id         the id of the person.
	 * @param fiscalCode the fiscal code of the person.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Person Working Time, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/personworkingtime/byPerson")
	public ResponseEntity<EPASPersonWorkingTimeDTO> getEPASChildrenByPerson(@RequestParam("id") Optional<String> id,
			@RequestParam("fiscalCode") Optional<String> fiscalCode) {

		EPASPersonWorkingTimeDTO epasPersonWorkingTime = null;
		if (id.isPresent()) {
			logger.info("REST request to get ePAS Person Working Time by Person id: {}", id.get());
			epasPersonWorkingTime = epasPersonWorkingTimeService.getByPersonId(id.get());
		} else {
			if (fiscalCode.isPresent()) {
				logger.info("REST request to get  ePAS Person Working Time by Person fiscalcode: {}", fiscalCode.get());
				epasPersonWorkingTime = epasPersonWorkingTimeService.getByPersonFiscalCode(fiscalCode.get());
			} else {
				return ResponseUtil.wrapOrNotFound(Optional.of(epasPersonWorkingTime),
						HeaderUtil.createFailureAlert(applicationName, false, "", "", "Invalid parameter in call"));
			}
		}
		logger.info("Retrieved EPAS Person Working Time: {}", epasPersonWorkingTime);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasPersonWorkingTime));
	}


	/**
	 * {@code GET /personworkingtime/list} : get personwokingtime by office id.
	 *
	 * @param officeId the office Id.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the list of EPAS Person Working Time, or with status
	 *         {@code 404 (Not Found)}.
	 */
	@GetMapping("/personworkingtime/list")
	public ResponseEntity<List<EPASPersonWorkingTimeDTO>> getEPASPersonWorkingTimeOfficeId(
			@RequestParam("officeId") String officeId) {
		logger.info("REST request to get ePAS Person Working Time by office Id={}", officeId);
		List<EPASPersonWorkingTimeDTO> epasPersonWorkingTimeList = epasPersonWorkingTimeService.getList(officeId);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasPersonWorkingTimeList));

	}

}
