package it.cnr.isti.epasmed.web.rest.epas;

import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.epas.model.EPASTimeCards;
import it.cnr.isti.epasmed.epas.service.EPASTimeCardsService;

@RestController
@RequestMapping("/api/epas")
public class EPASTimeCardsResource {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final EPASTimeCardsService epasTimeCardsService;

	public EPASTimeCardsResource(EPASTimeCardsService epasTimeCardsService) {
		this.epasTimeCardsService = epasTimeCardsService;

	}
	
	/**
	 * {@code GET /timecards/byPerson} : get time card by person, year and month
	 *
	 * @param id the id of the person.
	 * @param fiscalCode the fiscal code of the person.
	 * @param email the email of the person.
	 * @param year the year.
	 * @param month the month.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS TimeCards, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/timecards/byPerson")
	public ResponseEntity<EPASTimeCards> getEPASTimeCardsByPerson(@RequestParam("id") Optional<String> id,
			@RequestParam("fiscalCode") Optional<String> fiscalCode,
			@RequestParam("email") Optional<String> email,
			@RequestParam("year") String year,@RequestParam("month") String month) {
		
		
		EPASTimeCards epasTimeCards = null;
		if (id.isPresent()) {
			logger.info("REST request to get ePAS TimeCards by Person id: {}", id.get());
			epasTimeCards = epasTimeCardsService.getTimeCardByPersonId(id.get(), year, month);
		} else {
			if (fiscalCode.isPresent()) {
				logger.info("REST request to get ePAS TimeCards by Person fiscalcode: {}", fiscalCode.get());
				epasTimeCards = epasTimeCardsService.getTimeCardByPersonFiscalCode(fiscalCode.get(), year, month);
			} else {
				if (email.isPresent()) {
					logger.info("REST request to get ePAS TimeCards by Person email: {}", email.get());
					epasTimeCards = epasTimeCardsService.getTimeCardByPersonEmail(email.get(), year, month);
				} else {
					return ResponseUtil.wrapOrNotFound(Optional.of(epasTimeCards), HeaderUtil.createFailureAlert(applicationName,false,
							"","","Invalid parameter in call"));
				
				}
			}
		}
		logger.info("Retrieved Certification: {}", epasTimeCards);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasTimeCards));
	}

	
	
	
	/**
	 * {@code GET /timecards/byOffice} : get the time cards list.
	 *
	 * @param officeCodeId the id of the office.
	 * @param year the year
	 * @param month the month
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Time Card, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/timecards/byOffice")
	public ResponseEntity<List<EPASTimeCards>> getEPASTimeCardByOffice(@RequestParam("codeId") String id,
			@RequestParam("year") String year, @RequestParam("month") String month) {
		logger.info("REST request to get ePAS TimeCard list: officeCodeId={}, year={}, month={}", id,year,month);
		List<EPASTimeCards> epasTimeCardsList = epasTimeCardsService.getTimeCardByOfficeCodeId(id, year, month);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasTimeCardsList));

	}
	
}
