package it.cnr.isti.epasmed.web.rest.epas;

import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.epas.model.EPASValidates;
import it.cnr.isti.epasmed.epas.service.EPASValidatesService;

@RestController
@RequestMapping("/api/epas")
public class EPASValidatesResource {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final EPASValidatesService epasValidatesService;

	public EPASValidatesResource(EPASValidatesService epasValidatesService) {
		this.epasValidatesService = epasValidatesService;

	}

	/**
	 * {@code GET /validates/is} : return true if is validates the certification.
	 * 
	 * @param email the email of the person.
	 * @param year the year.
	 * @param month the month.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the Boolean value, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/validates/is")
	public ResponseEntity<String> isValidatesByPersonEmail(@RequestParam("email") String email,
			@RequestParam("year") String year, @RequestParam("month") String month) {
		logger.info("REST request to ePAS if is Validates by Person Email: email={}, year={}, month={}", email,year,month);
		String validates = epasValidatesService.isValidatesByPersonEmail(email, year, month);
		logger.info("Retrieved Validates: {}", validates);
		return ResponseUtil.wrapOrNotFound(Optional.of(validates));

	}
	
	
	/**
	 * {@code GET /validates} : get the list of certification validates.
	 *
	 * @param officeCodeId the id of the office.
	 * @param year the year.
	 * @param month the month.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS Validates, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/validates")
	public ResponseEntity<EPASValidates> getEPASValidatesByOfficeCodeId(@RequestParam("officeCodeId") String officeCodeId,
			@RequestParam("year") String year, @RequestParam("month") String month) {
		logger.info("REST request to get ePAS TimeCard list: officeCodeId={}, year={}, month={}", officeCodeId,year,month);
		EPASValidates epasValidates = epasValidatesService.getValidatesByOfficeCodeId(officeCodeId, year, month);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasValidates));

	}
	
}
