package it.cnr.isti.epasmed.web.rest.epas;

import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import io.github.jhipster.web.util.ResponseUtil;
import it.cnr.isti.epasmed.epas.model.EPASWorkingTimeTypes;
import it.cnr.isti.epasmed.epas.service.EPASWorkingTimeTypesService;

@RestController
@RequestMapping("/api/epas")
public class EPASWorkingTimeTypesResource {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	@Value("${jhipster.clientApp.name}")
	private String applicationName;

	private final EPASWorkingTimeTypesService epasWorkingTimeTypesService;

	public EPASWorkingTimeTypesResource(EPASWorkingTimeTypesService epasWorkingTimeTypesServiceService) {
		this.epasWorkingTimeTypesService = epasWorkingTimeTypesServiceService;

	}

	/**
	 * {@code GET /workingtimetypes/:id} : get workingtimetypes by id.
	 *
	 * @param id the id of the workingtimetypes to find.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the EPAS WorkingTimeTypes, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/workingtimetypes/{id}")
	public ResponseEntity<EPASWorkingTimeTypes> getEPASWorkingTimeTypesById(@PathVariable String id) {
		logger.info("REST request to get ePAS WorkingTimeTypes by Id: {}", id);
		EPASWorkingTimeTypes epasWorkingTimeTypes = epasWorkingTimeTypesService.getById(id);
		logger.info("Retrieved WorkingTimeTypes: {}", epasWorkingTimeTypes);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasWorkingTimeTypes));

	}

	/**
	 * {@code GET /workingtimetypes} : get the workingtimetypes list.
	 *
	 * @param officeCodeId the office codeId.
	 * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body
	 *         the list of EPASWorkingTimeTypes, or with status {@code 404 (Not Found)}.
	 */
	@GetMapping("/workingtimetypes")
	public ResponseEntity<List<EPASWorkingTimeTypes>> getEPASWorkingTimeTypes(@RequestParam("officeCodeId") String officeCodeId) {
		logger.info("REST request to get ePAS WorkingTimeTypes list for Office: {}", officeCodeId);
		List<EPASWorkingTimeTypes> epasWorkingTimeTypesList = epasWorkingTimeTypesService.getListByOfficeCodeId(officeCodeId);
		return ResponseUtil.wrapOrNotFound(Optional.of(epasWorkingTimeTypesList));

	}

}
