import { Component, OnInit } from '@angular/core';
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';

import { Leaves } from './leaves.model';
import { LeavesService } from './leaves.service';

@Component({
  selector: 'jhi-leaves-update',
  templateUrl: './leaves-update.component.html',
})
export class LeavesUpdateComponent implements OnInit {
  leaves!: Leaves;
  isSaving = false;

  editForm = this.fb.group({
    id: [],
    idPersona: ['', [Validators.required]],
    cf: ['', [Validators.required]],
    codiceAssenzaDescrizione: ['', [Validators.required]],
    dataInizio: ['', [Validators.required]],
    dataFine: [''],
    codiceAssenzaCodice: ['', [Validators.required]],
    codiceAssenzaCnr: ['', [Validators.required]],
    durata: ['', [Validators.required]],
    dataMod: ['', [Validators.required]],
  });

  constructor(private leavesService: LeavesService, private route: ActivatedRoute, private fb: FormBuilder) {}

  ngOnInit(): void {
    this.route.data.subscribe(({ leaves }) => {
      if (leaves) {
        this.leaves = leaves;
        this.updateForm(leaves);
      }
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    this.updateLeaves(this.leaves);
    if (this.leaves.id !== undefined) {
      this.leavesService.update(this.leaves).subscribe(
        () => this.onSaveSuccess(),
        () => this.onSaveError()
      );
    } else {
      this.leavesService.create(this.leaves).subscribe(
        () => this.onSaveSuccess(),
        () => this.onSaveError()
      );
    }
  }

  private updateForm(leaves: Leaves): void {
    this.editForm.patchValue({
      id: leaves.id,
      idPersona: leaves.idPersona,
      cf: leaves.cf,
      codiceAssenzaDescrizione: leaves.codiceAssenzaDescrizione,
      dataInizio: leaves.dataInizio,
      dataFine: leaves.dataFine,
      codiceAssenzaCodice: leaves.codiceAssenzaCodice,
      codiceAssenzaCnr: leaves.codiceAssenzaCnr,
      durata: leaves.durata,
      dataMod: leaves.dataMod,
    });
  }

  private updateLeaves(leaves: Leaves): void {
    leaves.idPersona = this.editForm.get(['idPersona'])!.value;
    leaves.cf = this.editForm.get(['cf'])!.value;
    leaves.codiceAssenzaDescrizione = this.editForm.get(['codiceAssenzaDescrizione'])!.value;
    leaves.dataInizio = this.editForm.get(['dataInizio'])!.value;
    leaves.dataFine = this.editForm.get(['dataFine'])!.value;
    leaves.codiceAssenzaCodice = this.editForm.get(['codiceAssenzaCodice'])!.value;
    leaves.codiceAssenzaCnr = this.editForm.get(['codiceAssenzaCnr'])!.value;
    leaves.durata = this.editForm.get(['durata'])!.value;
    leaves.dataMod = this.editForm.get(['dataMod'])!.value;
  }

  private onSaveSuccess(): void {
    this.isSaving = false;
    this.previousState();
  }

  private onSaveError(): void {
    this.isSaving = false;
  }
}
