import { Injectable } from '@angular/core';
import { Resolve, ActivatedRouteSnapshot, Routes } from '@angular/router';
import { Observable, of } from 'rxjs';

import { Leaves, ILeaves } from './leaves.model';
import { LeavesService } from './leaves.service';
import { LeavesComponent } from './leaves.component';
import { LeavesDetailComponent } from './leaves-detail.component';
import { LeavesUpdateComponent } from './leaves-update.component';

@Injectable({ providedIn: 'root' })
export class LeavesResolve implements Resolve<ILeaves> {
  constructor(private service: LeavesService) {}

  resolve(route: ActivatedRouteSnapshot): Observable<ILeaves> {
    const id = route.params['id'];
    if (id) {
      return this.service.find(id);
    }
    return of(new Leaves());
  }
}

export const leavesRoute: Routes = [
  {
    path: '',
    component: LeavesComponent,
    data: {
      defaultSort: 'id,asc',
    },
  },
  {
    path: ':id/view',
    component: LeavesDetailComponent,
    resolve: {
      leaves: LeavesResolve,
    },
  },
  {
    path: 'new',
    component: LeavesUpdateComponent,
    resolve: {
      leaves: LeavesResolve,
    },
  },
  {
    path: ':id/edit',
    component: LeavesUpdateComponent,
    resolve: {
      leaves: LeavesResolve,
    },
  },
];
