import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';

import { SERVER_API_URL } from 'app/app.constants';
import { createRequestOption, Pagination } from 'app/shared/util/request-util';
import { ILeaves } from './leaves.model';

@Injectable({ providedIn: 'root' })
export class LeavesService {
  public resourceUrl = SERVER_API_URL + 'api/leaves';

  constructor(private http: HttpClient) {}

  query(req?: Pagination): Observable<HttpResponse<ILeaves[]>> {
    const options = createRequestOption(req);
    return this.http.get<ILeaves[]>(this.resourceUrl, { params: options, observe: 'response' });
  }

  create(leave: ILeaves): Observable<ILeaves> {
    return this.http.post<ILeaves>(this.resourceUrl, leave);
  }

  update(leave: ILeaves): Observable<ILeaves> {
    return this.http.put<ILeaves>(this.resourceUrl, leave);
  }

  find(id: string): Observable<ILeaves> {
    return this.http.get<ILeaves>(`${this.resourceUrl}/${id}`);
  }

  delete(id: string): Observable<{}> {
    return this.http.delete(`${this.resourceUrl}/${id}`);
  }
}
