import { Component, OnInit } from '@angular/core';
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';

import { PersOrario } from './persorario.model';
import { PersOrarioService } from './persorario.service';

@Component({
  selector: 'jhi-persorario-update',
  templateUrl: './persorario-update.component.html',
})
export class PersOrarioUpdateComponent implements OnInit {
  persOrario!: PersOrario;
  isSaving = false;

  editForm = this.fb.group({
    id: [],
    idPersona: ['', [Validators.required]],
    cf: ['', [Validators.required]],
    dal: ['', [Validators.required]],
    al: [''],
    descrizione: ['', [Validators.required]],
    lun: [''],
    mar: [''],
    mer: [''],
    gio: [''],
    ven: [''],
    sab: [''],
    percentuale: [''],
    turno: [''],
    oreTurno: [''],
    festivo: [''],
    notturno: [''],
    dataMod: ['', [Validators.required]],
  });

  constructor(private persOrarioService: PersOrarioService, private route: ActivatedRoute, private fb: FormBuilder) {}

  ngOnInit(): void {
    this.route.data.subscribe(({ persOrario }) => {
      if (persOrario) {
        this.persOrario = persOrario;
        this.updateForm(persOrario);
      }
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    this.updatePersOrario(this.persOrario);
    if (this.persOrario.id !== undefined) {
      this.persOrarioService.update(this.persOrario).subscribe(
        () => this.onSaveSuccess(),
        () => this.onSaveError()
      );
    } else {
      this.persOrarioService.create(this.persOrario).subscribe(
        () => this.onSaveSuccess(),
        () => this.onSaveError()
      );
    }
  }

  private updateForm(persOrario: PersOrario): void {
    this.editForm.patchValue({
      id: persOrario.id,
      idPersona: persOrario.idPersona,
      cf: persOrario.cf,
      dal: persOrario.dal,
      al: persOrario.al,
      descrizione: persOrario.descrizione,
      lun: persOrario.lun,
      mar: persOrario.mar,
      mer: persOrario.mer,
      gio: persOrario.gio,
      ven: persOrario.ven,
      sab: persOrario.sab,
      percentuale: persOrario.percentuale,
      turno: persOrario.turno,
      oreTurno: persOrario.oreTurno,
      festivo: persOrario.festivo,
      notturno: persOrario.notturno,
      dataMod: persOrario.dataMod,
    });
  }

  private updatePersOrario(persOrario: PersOrario): void {
    persOrario.idPersona = this.editForm.get(['idPersona'])!.value;
    persOrario.cf = this.editForm.get(['cf'])!.value;
    persOrario.dal = this.editForm.get(['dal'])!.value;
    persOrario.al = this.editForm.get(['al'])!.value;
    persOrario.descrizione = this.editForm.get(['descrizione'])!.value;
    persOrario.lun = this.editForm.get(['lun'])!.value;
    persOrario.mar = this.editForm.get(['mar'])!.value;
    persOrario.mer = this.editForm.get(['mer'])!.value;
    persOrario.gio = this.editForm.get(['gio'])!.value;
    persOrario.ven = this.editForm.get(['ven'])!.value;
    persOrario.sab = this.editForm.get(['sab'])!.value;
    persOrario.percentuale = this.editForm.get(['percentuale'])!.value;
    persOrario.turno = this.editForm.get(['turno'])!.value;
    persOrario.oreTurno = this.editForm.get(['oreTurno'])!.value;
    persOrario.festivo = this.editForm.get(['festivo'])!.value;
    persOrario.notturno = this.editForm.get(['notturno'])!.value;
    persOrario.dataMod = this.editForm.get(['dataMod'])!.value;
  }

  private onSaveSuccess(): void {
    this.isSaving = false;
    this.previousState();
  }

  private onSaveError(): void {
    this.isSaving = false;
  }
}
