import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';

import { SERVER_API_URL } from 'app/app.constants';
import { createRequestOption, Pagination } from 'app/shared/util/request-util';
import { IPersOrario } from './persorario.model';

@Injectable({ providedIn: 'root' })
export class PersOrarioService {
  public resourceUrl = SERVER_API_URL + 'api/persorario';

  constructor(private http: HttpClient) {}

  query(req?: Pagination): Observable<HttpResponse<IPersOrario[]>> {
    const options = createRequestOption(req);
    return this.http.get<IPersOrario[]>(this.resourceUrl, { params: options, observe: 'response' });
  }

  create(persOrario: IPersOrario): Observable<IPersOrario> {
    return this.http.post<IPersOrario>(this.resourceUrl, persOrario);
  }

  update(persOrario: IPersOrario): Observable<IPersOrario> {
    return this.http.put<IPersOrario>(this.resourceUrl, persOrario);
  }

  find(id: string): Observable<IPersOrario> {
    return this.http.get<IPersOrario>(`${this.resourceUrl}/${id}`);
  }

  delete(id: string): Observable<{}> {
    return this.http.delete(`${this.resourceUrl}/${id}`);
  }
}
