import { Component, OnInit } from '@angular/core';
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';

import { TabsSI } from './tabssi.model';
import { TabsSIService } from './tabssi.service';

@Component({
  selector: 'jhi-tabssi-update',
  templateUrl: './tabssi-update.component.html',
})
export class TabsSIUpdateComponent implements OnInit {
  tabsSI!: TabsSI;
  isSaving = false;

  editForm = this.fb.group({
    id: [],
    nome: ['', [Validators.required]],
    operazioni: ['', [Validators.required]],
    idFlusso: ['', [Validators.required]],
    lastUpdate: ['', [Validators.required]],
  });

  constructor(private tabsSIService: TabsSIService, private route: ActivatedRoute, private fb: FormBuilder) {}

  ngOnInit(): void {
    this.route.data.subscribe(({ tabsSI }) => {
      if (tabsSI) {
        this.tabsSI = tabsSI;
        this.updateForm(tabsSI);
      }
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    this.updateTabsSI(this.tabsSI);
    if (this.tabsSI.id !== undefined) {
      this.tabsSIService.update(this.tabsSI).subscribe(
        () => this.onSaveSuccess(),
        () => this.onSaveError()
      );
    } else {
      this.tabsSIService.create(this.tabsSI).subscribe(
        () => this.onSaveSuccess(),
        () => this.onSaveError()
      );
    }
  }

  private updateForm(tabsSI: TabsSI): void {
    this.editForm.patchValue({
      id: tabsSI.id,
      nome: tabsSI.nome,
      operazioni: tabsSI.operazioni,
      idFlusso: tabsSI.idFlusso,
      lastUpdate: tabsSI.lastUpdate,
    });
  }

  private updateTabsSI(tabsSI: TabsSI): void {
    tabsSI.nome = this.editForm.get(['nome'])!.value;
    tabsSI.operazioni = this.editForm.get(['operazioni'])!.value;
    tabsSI.idFlusso = this.editForm.get(['idFlusso'])!.value;
    tabsSI.lastUpdate = this.editForm.get(['lastUpdate'])!.value;
  }

  private onSaveSuccess(): void {
    this.isSaving = false;
    this.previousState();
  }

  private onSaveError(): void {
    this.isSaving = false;
  }
}
