import { Injectable } from '@angular/core';
import { Resolve, ActivatedRouteSnapshot, Routes } from '@angular/router';
import { Observable, of } from 'rxjs';

import { TabsSI, ITabsSI } from './tabssi.model';
import { TabsSIService } from './tabssi.service';
import { TabsSIComponent } from './tabssi.component';
import { TabsSIDetailComponent } from './tabssi-detail.component';
import { TabsSIUpdateComponent } from './tabssi-update.component';

@Injectable({ providedIn: 'root' })
export class TabsSIResolve implements Resolve<ITabsSI> {
  constructor(private service: TabsSIService) {}

  resolve(route: ActivatedRouteSnapshot): Observable<ITabsSI> {
    const id = route.params['id'];
    if (id) {
      return this.service.find(id);
    }
    return of(new TabsSI());
  }
}

export const tabsSIRoute: Routes = [
  {
    path: '',
    component: TabsSIComponent,
    data: {
      defaultSort: 'id,asc',
    },
  },
  {
    path: ':id/view',
    component: TabsSIDetailComponent,
    resolve: {
      tabsSI: TabsSIResolve,
    },
  },
  {
    path: 'new',
    component: TabsSIUpdateComponent,
    resolve: {
      tabsSI: TabsSIResolve,
    },
  },
  {
    path: ':id/edit',
    component: TabsSIUpdateComponent,
    resolve: {
      tabsSI: TabsSIResolve,
    },
  },
];
