import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';

import { SERVER_API_URL } from 'app/app.constants';
import { createRequestOption, Pagination } from 'app/shared/util/request-util';
import { ITabsSI } from './tabssi.model';

@Injectable({ providedIn: 'root' })
export class TabsSIService {
  public resourceUrl = SERVER_API_URL + 'api/tabssi';

  constructor(private http: HttpClient) {}

  query(req?: Pagination): Observable<HttpResponse<ITabsSI[]>> {
    const options = createRequestOption(req);
    return this.http.get<ITabsSI[]>(this.resourceUrl, { params: options, observe: 'response' });
  }

  create(tabssi: ITabsSI): Observable<ITabsSI> {
    return this.http.post<ITabsSI>(this.resourceUrl, tabssi);
  }

  update(tabssi: ITabsSI): Observable<ITabsSI> {
    return this.http.put<ITabsSI>(this.resourceUrl, tabssi);
  }

  find(id: string): Observable<ITabsSI> {
    return this.http.get<ITabsSI>(`${this.resourceUrl}/${id}`);
  }

  delete(id: string): Observable<{}> {
    return this.http.delete(`${this.resourceUrl}/${id}`);
  }
}
