import { Component, OnInit } from '@angular/core';
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';

import { TimeCardsReporting } from './timecardsreporting.model';
import { TimeCardsReportingService } from './timecardsreporting.service';

@Component({
  selector: 'jhi-timecardsreporting-update',
  templateUrl: './timecardsreporting-update.component.html',
})
export class TimeCardsReportingUpdateComponent implements OnInit {
  timeCardsReporting!: TimeCardsReporting;
  isSaving = false;

  editForm = this.fb.group({
    id: [],
    year: ['', [Validators.required]],
    month: ['', [Validators.required]],
    idFlusso: ['', [Validators.required]],
    lastUpdate: ['', [Validators.required]],
  });

  constructor(private timeCardsReportingService: TimeCardsReportingService, private route: ActivatedRoute, private fb: FormBuilder) {}

  ngOnInit(): void {
    this.route.data.subscribe(({ timeCardsReporting }) => {
      if (timeCardsReporting) {
        this.timeCardsReporting = timeCardsReporting;
        this.updateForm(timeCardsReporting);
      }
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    this.updateTimeCardsReporting(this.timeCardsReporting);
    if (this.timeCardsReporting.id !== undefined) {
      this.timeCardsReportingService.update(this.timeCardsReporting).subscribe(
        () => this.onSaveSuccess(),
        () => this.onSaveError()
      );
    } else {
      this.timeCardsReportingService.create(this.timeCardsReporting).subscribe(
        () => this.onSaveSuccess(),
        () => this.onSaveError()
      );
    }
  }

  private updateForm(timeCardsReporting: TimeCardsReporting): void {
    this.editForm.patchValue({
      id: timeCardsReporting.id,
      year: timeCardsReporting.year,
      month: timeCardsReporting.month,
      idFlusso: timeCardsReporting.idFlusso,
      lastUpdate: timeCardsReporting.lastUpdate,
    });
  }

  private updateTimeCardsReporting(timeCardsReporting: TimeCardsReporting): void {
    timeCardsReporting.year = this.editForm.get(['year'])!.value;
    timeCardsReporting.month = this.editForm.get(['month'])!.value;
    timeCardsReporting.idFlusso = this.editForm.get(['idFlusso'])!.value;
    timeCardsReporting.lastUpdate = this.editForm.get(['lastUpdate'])!.value;
  }

  private onSaveSuccess(): void {
    this.isSaving = false;
    this.previousState();
  }

  private onSaveError(): void {
    this.isSaving = false;
  }
}
