import { Component, OnInit, AfterViewInit, ViewChild, ElementRef } from '@angular/core';
import { EPASPerson } from './rendicontazionesingolo.model';
import { RendicontazioneSingoloService } from './rendicontazionesingolo.service';
import { HttpErrorResponse } from '@angular/common/http';
// import { Observable, OperatorFunction } from 'rxjs';
// import { debounceTime, distinctUntilChanged, map, filter } from 'rxjs/operators';
import { NgbModal, NgbModalRef } from '@ng-bootstrap/ng-bootstrap';

@Component({
  selector: 'jhi-rendicontazionesingolo',
  templateUrl: './rendicontazionesingolo.component.html',
  styleUrls: ['rendicontazionesingolo.scss'],
})
export class RendicontazioneSingoloComponent implements OnInit, AfterViewInit {
  year: number;
  month: number;
  fiscalCode: string;
  dipendenti: EPASPerson[];
  // isLoading = false; // hidden by default
  m!: NgbModalRef;
  @ViewChild('content', { static: true }) content!: ElementRef;

  constructor(private modalService: NgbModal, private rendicontazioneSingoloService: RendicontazioneSingoloService) {
    const date: Date = new Date();
    this.year = date.getFullYear();
    this.month = date.getMonth() + 1;
    this.fiscalCode = '';
    this.dipendenti = [];
    // this.content = {} as ElementRef;
  }

  ngOnInit(): void {}

  ngAfterViewInit(): void {
    this.openLoading();
    this.rendicontazioneSingoloService.getPersonList().subscribe(
      (result: EPASPerson[]) => {
        this.dipendenti = result;
        this.fiscalCode = this.dipendenti[0].fiscalCode;
        this.dipendenti.forEach(dip =>
          // eslint-disable-next-line no-console
          console.log(dip)
        );
        this.closeLoading();
      },
      error => {
        // eslint-disable-next-line no-console
        console.log(error);
        this.closeLoading();
      }
    );
  }

  private openLoading(): void {
    this.m = this.modalService.open(this.content, { size: 'sm', centered: true, backdrop: 'static', keyboard: false });
  }
  private closeLoading(): void {
    this.m.close();
    // this.modalService.dismissAll('Loaded');
  }

  // formatter = (dipendente: EPASPerson) => dipendente.fullname;

  // search: OperatorFunction<string, readonly EPASPerson[]> = (text$: Observable<string>) => text$.pipe(
  //	debounceTime(200),
  //	distinctUntilChanged(),
  //	filter(term => term.length >= 2),
  //	map(term => this.dipendenti.filter(dipendente => new RegExp(term, 'mi').test(dipendente.fullname.toLocaleLowerCase())).slice(0, 10))
  // );

  rendicontaSingolo(): void {
    this.openLoading();
    if (this.year == null || this.year <= 0) {
      // eslint-disable-next-line no-console
      console.log('Select the year');
      this.closeLoading();
      return;
    }
    if (this.month == null || this.month <= 0) {
      // eslint-disable-next-line no-console
      console.log('Select the month');
      this.closeLoading();
      return;
    }

    if (!this.fiscalCode) {
      // eslint-disable-next-line no-console
      console.log('Select the employee');
      this.closeLoading();
      return;
    }

    // eslint-disable-next-line no-console
    console.log(this.fiscalCode);

    this.rendicontazioneSingoloService.rendicontaSingolo(this.year, this.month, this.fiscalCode).subscribe(
      () => this.onSuccess(),
      error => this.onError(error)
    );
  }

  private onSuccess(): void {
    this.closeLoading();
    // eslint-disable-next-line no-console
    console.log('Success');
  }

  private onError(error: HttpErrorResponse): void {
    this.closeLoading();
    // eslint-disable-next-line no-console
    console.log('Error');
    // eslint-disable-next-line no-console
    console.log(error);
  }
}
