import { Component, OnInit, ViewChild, ElementRef } from '@angular/core';
// import { NgbDate } from '@ng-bootstrap/ng-bootstrap';
import { HttpErrorResponse } from '@angular/common/http';
import { NgbModal, NgbModalRef } from '@ng-bootstrap/ng-bootstrap';

import { SyncType } from './sync.model';
import { SyncService } from './sync.service';

@Component({
  selector: 'jhi-sync',
  templateUrl: './sync.component.html',
  styleUrls: ['sync.scss'],
})
export class SyncComponent implements OnInit {
  syncT: SyncType | null = null;
  year: number;
  month: number;

  m!: NgbModalRef;
  @ViewChild('content', { static: true }) content!: ElementRef;

  constructor(private modalService: NgbModal, private syncService: SyncService) {
    const date: Date = new Date();
    this.year = date.getFullYear();
    this.month = date.getMonth();
  }

  ngOnInit(): void {}

  private openLoading(): void {
    this.m = this.modalService.open(this.content, { size: 'sm', centered: true, backdrop: 'static', keyboard: false });
  }
  private closeLoading(): void {
    this.m.close();
    // this.modalService.dismissAll('Loaded');
  }

  sync(syncType: SyncType): void {
    this.openLoading();
    this.syncT = syncType;
    if (this.year == null || this.year <= 0) {
      this.closeLoading();
      // eslint-disable-next-line no-console
      console.log('Select the year');
      return;
    }
    if (this.month == null || this.month <= 0) {
      this.closeLoading();
      // eslint-disable-next-line no-console
      console.log('Select the month');
      return;
    }

    this.syncService.sync(syncType, this.year, this.month).subscribe(
      () => this.onSuccess(),
      error => this.onError(error)
    );
  }

  private onSuccess(): void {
    this.closeLoading();
    // eslint-disable-next-line no-console
    console.log('Success');
  }

  private onError(error: HttpErrorResponse): void {
    this.closeLoading();
    // eslint-disable-next-line no-console
    console.log('Error');
    // eslint-disable-next-line no-console
    console.log(error);

    //
  }
}
