package it.cnr.isti.epasmed.web.rest.epas;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.env.Environment;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.EnabledIf;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import it.cnr.isti.epasmed.EpasmedApp;
import it.cnr.isti.epasmed.epas.model.EPASAbsenceTypes;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;

/**
 * Integration tests for the {@link EPASAbsenceTypesResource} REST controller.
 */
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@SpringBootTest(classes = EpasmedApp.class)
@EnabledIf("false")
public class EPASAbsenceTypesResourceIT {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	// private static final String OFFICE_DEFAULT_ID = "1";
	// private static final String OFFICE_DEFAULT_NAME = "ISTI - Pisa";
	// private static final String OFFICE_DEFAULT_CODE = "074000";
	// private static final String OFFICE_DEFAULT_CODEID = "225200";

	@Autowired
	private MockMvc restEPASAbsenceTypesMockMvc;

	@Autowired
	private Environment environment;

	@BeforeEach
	public void initTest() {
		for (String profileName : environment.getActiveProfiles()) {
			logger.info("Currently active profile - " + profileName);
		}
	}

	@Test
	public void getAbsenceTypes() throws Exception {

		List<EPASAbsenceTypes> epasAbsenceTypes = null;
		try {
			MvcResult result = restEPASAbsenceTypesMockMvc.perform(get("/api/epas/absenceTypes"))
					.andExpect(status().isOk()).andReturn();

			ObjectMapper mapper = new ObjectMapper();
			epasAbsenceTypes = mapper.readValue(result.getResponse().getContentAsString(),
					new TypeReference<List<EPASAbsenceTypes>>() {
					});

		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
			return;
		}

		if (epasAbsenceTypes != null) {
			logger.info("EPAS Absence Types size: {}", epasAbsenceTypes.size());
			for (EPASAbsenceTypes type : epasAbsenceTypes) {
				logger.info("{}", type);
			}
		}

	}

	@Test
	public void getAbsenceTypesMap() throws Exception {

		LinkedHashMap<String, String> epasAbsenceTypesMap = null;
		try {
			MvcResult result = restEPASAbsenceTypesMockMvc.perform(get("/api/epas/absenceTypesMap"))
					.andExpect(status().isOk()).andReturn();

			ObjectMapper mapper = new ObjectMapper();
			epasAbsenceTypesMap = mapper.readValue(result.getResponse().getContentAsString(),
					new TypeReference<LinkedHashMap<String, String>>() {
					});

		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
			return;
		}

		if (epasAbsenceTypesMap != null) {
			logger.info("EPAS Absence Types Map size: {}", epasAbsenceTypesMap.size());
			for (Map.Entry<String, String> m : epasAbsenceTypesMap.entrySet()) {
				logger.info(m.getKey() + "-" + m.getValue());
			}

		}

	}

}
