package it.cnr.isti.epasmed.web.rest.epas;

import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.EnabledIf;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import it.cnr.isti.epasmed.EpasmedApp;
import it.cnr.isti.epasmed.epas.dto.EPASAffiliationsDTO;
import it.cnr.isti.epasmed.epas.mapper.EPASAffiliationsMapper;
import it.cnr.isti.epasmed.epas.model.EPASAffiliations;
import it.cnr.isti.epasmed.epas.model.EPASGroups;
import it.cnr.isti.epasmed.epas.model.EPASPersons;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.web.rest.TestUtil;

/**
 * Integration tests for the {@link EPASAffiliationsResource} REST controller.
 */
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@SpringBootTest(classes = EpasmedApp.class)
@EnabledIf("false")
public class EPASAffiliationsResourceIT {

	private final Logger log = LoggerFactory.getLogger(getClass());

	private static final String OFFICE_DEFAULT_ID = "1";
	//private static final String OFFICE_DEFAULT_NAME = "ISTI - Pisa";
	//private static final String OFFICE_DEFAULT_CODE = "074000";
	//private static final String OFFICE_DEFAULT_CODEID = "225200";

	private static final String AFFILIATION_DEFAULT_ID = "1";
	private static final String AFFILIATION_DEFAULT_BEGINDATE = "2021-01-01";
	private static final String AFFILIATION_DEFAULT_ENDDATE = null;
	private static final String AFFILIATION_DEFAULT_GROUPID = "1";
	private static final String AFFILIATION_DEFAULT_PERCENTAGE = "100";
	private static final String AFFILIATION_DEFAULT_PERSONID = "1";
	private static final String AFFILIATION_DEFAULT_EXTERNALID = "1";

	//private ArrayList<EPASAffiliationsDTO> epasAffiliationsDTOList;

	@Autowired
	private MockMvc restEPASAffiliationsMockMvc;

	@Autowired
	private EPASAffiliationsMapper epasAffiliationsMapper;

	@Autowired
	private Environment environment;

	//private EPASAffiliations epasAffiliations;

	private EPASAffiliationsDTO epasAffiliationsDTO;

	private EPASAffiliationsDTO createEPASAffiliationsDTO() {
		EPASAffiliationsDTO epasAffiliationsDTO = new EPASAffiliationsDTO();
		epasAffiliationsDTO.setBeginDate(AFFILIATION_DEFAULT_BEGINDATE);
		epasAffiliationsDTO.setEndDate(AFFILIATION_DEFAULT_ENDDATE);
		epasAffiliationsDTO.setGroupId(AFFILIATION_DEFAULT_GROUPID);
		epasAffiliationsDTO.setId(AFFILIATION_DEFAULT_ID);
		epasAffiliationsDTO.setPercentage(AFFILIATION_DEFAULT_PERCENTAGE);
		epasAffiliationsDTO.setPersonId(AFFILIATION_DEFAULT_PERSONID);
		epasAffiliationsDTO.setExternalId(AFFILIATION_DEFAULT_EXTERNALID);
		return epasAffiliationsDTO;
	}

	@BeforeEach
	public void initTest() {
		for (String profileName : environment.getActiveProfiles()) {
			log.info("Currently active profile - " + profileName);
		}
		epasAffiliationsDTO = createEPASAffiliationsDTO();
	}

	@Test
	public void getAffiliation() throws Exception {
		restEPASAffiliationsMockMvc.perform(get("/api/epas/affiliations/" + 1)).andExpect(status().isOk());
	}

	@Test
	public void getAffiliationByGroup() throws Exception {
		restEPASAffiliationsMockMvc.perform(get("/api/epas/affiliations/byGroup?id=" + 23)).andExpect(status().isOk());
	}

	@Test
	public void getAffiliationByPersonId() throws Exception {
		restEPASAffiliationsMockMvc.perform(get("/api/epas/affiliations/byPerson?id=" + 43)).andExpect(status().isOk());
	}

	@Test
	public void getAffiliationByPersonFiscalcode() throws Exception {
		restEPASAffiliationsMockMvc.perform(get("/api/epas/affiliations/byPerson?fiscalCode=DZZPRZ79S23B832W"))
				.andExpect(status().isOk());
	}

	@Test
	public void getAffiliationList() throws Exception {
		restEPASAffiliationsMockMvc.perform(get("/api/epas/affiliations?officeId=" + OFFICE_DEFAULT_ID))
				.andExpect(status().isOk());
	}

	@Test
	public void createAffiliation() throws Exception {
		restEPASAffiliationsMockMvc
				.perform(post("/api/epas/affiliations").contentType(MediaType.APPLICATION_JSON)
						.content(TestUtil.convertObjectToJsonBytes(epasAffiliationsDTO)).with(csrf()))
				.andExpect(status().isCreated());
	}

	@Test
	public void istiCreateAffiliations() throws Exception {

		String userDirectory = System.getProperty("user.dir");
		log.info(userDirectory);
		List<EPASAffiliations> istiAffiliations = null;
		try (Stream<String> stream = Files
				.lines(Paths.get("src/test/resources/it/cnr/isti/epasmed/web/rest/epas/affiliazioniRepartiISTIRimanenti.csv"))) {
			istiAffiliations = stream.skip(1).map(mapToAffiliation).collect(Collectors.toList());
		} catch (Exception e) {
			log.error(e.getLocalizedMessage(), e);
			return;
		}

		log.info("ISTI Persons loaded");

		List<EPASAffiliations> pISTIAffiliations = new ArrayList<>();
		for (EPASAffiliations a : istiAffiliations) {
			if (a.getPerson() != null && a.getPerson().getFiscalCode() != null
					&& !a.getPerson().getFiscalCode().isEmpty()) {
				a.setPerson(retrievePersonByFiscalCode(a.getPerson().getFiscalCode()));
				pISTIAffiliations.add(a);

			}

		}

		log.info("Added to ISTI Affiliations the EPAS Persons: {}", pISTIAffiliations.size());

		List<EPASGroups> epasGroups = retrieveEPASGroups();
		log.info("EPAS Groups loaded");
		
		List<EPASAffiliations> gISTIAffiliations = new ArrayList<>();
		for (EPASAffiliations a : pISTIAffiliations) {
			if (a.getGroup() != null && a.getGroup().getExternalId() != null
					&& !a.getGroup().getExternalId().isEmpty()) {
				EPASGroups g = retrieveEPASGroupByExternalId(epasGroups, a.getGroup().getExternalId());
				if (g != null) {
					a.setGroup(g);
					gISTIAffiliations.add(a);
				}
			}
		}

		log.info("Added to ISTI Affiliations the EPAS Groups: "+gISTIAffiliations.size());

	
		List<EPASAffiliationsDTO> istiAffList = epasAffiliationsMapper
				.epasAffiliationsToEPASAffiliationsDTOs(gISTIAffiliations);
		
		
		
		for (EPASAffiliationsDTO ia : istiAffList) {
			log.info("Isti Aff: {}",ia);
			restEPASAffiliationsMockMvc
					.perform(post("/api/epas/affiliations").contentType(MediaType.APPLICATION_JSON)
							.content(TestUtil.convertObjectToJsonBytes(ia)).with(csrf()))
					.andExpect(status().is2xxSuccessful());
		}

		log.info("Added to ISTI Affiliations");
	
	}

	private EPASGroups retrieveEPASGroupByExternalId(List<EPASGroups> epasGroups, String externalId) {
		for (EPASGroups g : epasGroups) {
			if (g.getExternalId() != null && !g.getExternalId().isEmpty()
					&& g.getExternalId().compareTo(externalId) == 0) {
				return g;
			}
		}
		return null;
	}

	private List<EPASGroups> retrieveEPASGroups() throws Exception {
		MvcResult result = restEPASAffiliationsMockMvc.perform(get("/api/epas/groups?officeId=" + OFFICE_DEFAULT_ID))
				.andExpect(status().isOk()).andReturn();
		ObjectMapper mapper = new ObjectMapper();
		List<EPASGroups> epasGroups = mapper.readValue(result.getResponse().getContentAsString(),
				new TypeReference<List<EPASGroups>>() {
				});
		return epasGroups;
	}

	private EPASPersons retrievePersonByFiscalCode(String fiscalCode) throws Exception {
		MvcResult result = restEPASAffiliationsMockMvc.perform(get("/api/epas/persons/show?fiscalCode=" + fiscalCode))
				.andExpect(status().isOk()).andReturn();

		ObjectMapper mapper = new ObjectMapper();
		EPASPersons epasPersons = mapper.readValue(result.getResponse().getContentAsString(),
				new TypeReference<EPASPersons>() {
				});
		return epasPersons;
	}

	private Function<String, EPASAffiliations> mapToAffiliation = (line) -> {

		String[] a = line.split(",");// a CSV has comma separated lines

		EPASAffiliations aff = new EPASAffiliations();

		// <-- this is the first column in the csv file
		if (a[0] != null && a[0].length() > 0) {
			String externalId = a[0];
			if (externalId != null && !externalId.isEmpty()) {
				aff.setExternalId(externalId);
			}
		}

		EPASGroups g = new EPASGroups();
		if (a[1] != null && a[1].length() > 0) {
			String groupId = a[1];
			if (groupId != null && !groupId.isEmpty()) {
				g.setExternalId(groupId);

			}
		}
		aff.setGroup(g);

		EPASPersons pe = new EPASPersons();
		/*
		 * if (a[2] != null && a[2].length() > 0) { String personId = a[2]; if (personId
		 * != null && !personId.isEmpty()) {
		 * 
		 * } }
		 */

		if (a[3] != null && a[3].length() > 0) {
			String fiscalCode = a[3].substring(1, a[3].length() - 1);
			if (fiscalCode != null && !fiscalCode.isEmpty()) {
				pe.setFiscalCode(fiscalCode);
			}
		}
		aff.setPerson(pe);

		if (a[4] != null && a[4].length() > 0) {
			String beginDate = a[4].substring(1, a[4].length() - 1);
			if (beginDate != null && !beginDate.isEmpty()) {
				aff.setBeginDate(beginDate.substring(0, 10));
			}
		}

		if (a[5] != null && a[5].length() > 0) {
			String endDate = a[5].substring(1, a[5].length() - 1);
			if (endDate != null && !endDate.isEmpty()) {
				aff.setEndDate(endDate.substring(0, 10));
			}
		}

		if (a[6] != null && a[6].length() > 0) {
			String percentage = a[6];
			if (percentage != null && !percentage.isEmpty()) {
				aff.setPercentage(percentage);
			}
		}

		return aff;
	};

	@Test
	public void updateAffiliationById() throws Exception {
		epasAffiliationsDTO.setPercentage("50");
		MvcResult mvcResult = restEPASAffiliationsMockMvc
				.perform(put("/api/epas/affiliations/" + AFFILIATION_DEFAULT_ID).contentType(MediaType.APPLICATION_JSON)
						.content(TestUtil.convertObjectToJsonBytes(epasAffiliationsDTO)).with(csrf()))
				.andExpect(status().isOk()).andReturn();
		ObjectMapper mapper = new ObjectMapper();

		EPASAffiliations epasAffiliations = mapper.readValue(mvcResult.getResponse().getContentAsByteArray(),
				EPASAffiliations.class);
		log.info("Updated EPAS Affiliation by id: " + epasAffiliations);
	}

	@Test
	public void deleteAffiliationById() throws Exception {
		restEPASAffiliationsMockMvc.perform(delete("/api/epas/affiliations/" + AFFILIATION_DEFAULT_ID).with(csrf()))
				.andExpect(status().is2xxSuccessful());
		log.info("Deleted EPAS Affiliation by id: " + AFFILIATION_DEFAULT_ID);
	}

}
