package it.cnr.isti.epasmed.web.rest.epas;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.env.Environment;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.EnabledIf;
import org.springframework.test.web.servlet.MockMvc;

import it.cnr.isti.epasmed.EpasmedApp;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;

/**
 * Integration tests for the {@link EPASCertificationsResource} REST controller.
 */
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@SpringBootTest(classes = EpasmedApp.class)
@EnabledIf("false")
public class EPASCertificationsResourceIT {

	private final Logger log = LoggerFactory.getLogger(getClass());

	// private static final String OFFICE_DEFAULT_ID = "1";
	// private static final String OFFICE_DEFAULT_NAME = "ISTI - Pisa";
	// private static final String OFFICE_DEFAULT_CODE = "074000";
	private static final String OFFICE_DEFAULT_CODEID = "225200";

	private static final String PERSON_DEFAULT_ID = "113";
	//private static final String PERSON_DEFAULT_NAME = "Giuseppe";
	//private static final String PERSON_DEFAULT_SURNAME = "Amato";
	private static final String PERSON_DEFAULT_FISCAL_CODE = "MTAGPP68D15D976W";
	private static final String PERSON_DEFAULT_EMAIL = "giuseppe.amato@cnr.it";
	//private static final String PERSON_DEFAULT_QUALIFICATION = "2";
	
	
	@Autowired
	private MockMvc restEPASCertificationsMockMvc;

	@Autowired
	private Environment environment;

	//private EPASLeaves epasLeaves;

	@BeforeEach
	public void initTest() {
		for (String profileName : environment.getActiveProfiles()) {
			log.info("Currently active profile - " + profileName);
		}
	}

	@Test
	public void getEPASCertificationsByPersonId() throws Exception {
		restEPASCertificationsMockMvc.perform(get("/api/epas/certifications/getMonthSituation?id=" + PERSON_DEFAULT_ID + "&year=2021&month=11"))
				.andExpect(status().isOk());
	}
	
	@Test
	public void getEPASCertificationsByPersonFiscalcode() throws Exception {
		restEPASCertificationsMockMvc
				.perform(
						get("/api/epas/certifications/getMonthSituation?fiscalCode=" + PERSON_DEFAULT_FISCAL_CODE + "&year=2021&month=11"))
				.andExpect(status().isOk());
	}

	@Test
	public void getEPASCertificationsByPersonEmail() throws Exception {
		restEPASCertificationsMockMvc
				.perform(get("/api/epas/certifications/getMonthSituation?email=" + PERSON_DEFAULT_EMAIL + "&year=2021&month=11"))
				.andExpect(status().isOk());
	}

	@Test
	public void getEPASCertificationsByOfficeCodeId() throws Exception {
		restEPASCertificationsMockMvc
				.perform(get("/api/epas/certifications/getMonthSituationByOffice?officeCodeId=" + OFFICE_DEFAULT_CODEID + "&year=2021&month=10"))
				.andExpect(status().isOk());
	}

}
