package it.cnr.isti.epasmed.web.rest.epas;

import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.io.UnsupportedEncodingException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.EnabledIf;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import it.cnr.isti.epasmed.EpasmedApp;
import it.cnr.isti.epasmed.epas.dto.EPASContractsDTO;
import it.cnr.isti.epasmed.epas.mapper.EPASContractsMapper;
import it.cnr.isti.epasmed.epas.model.EPASContracts;
import it.cnr.isti.epasmed.epas.model.EPASPersons;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.web.rest.TestUtil;

/**
 * Integration tests for the {@link EPASContractsResource} REST controller.
 */
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@SpringBootTest(classes = EpasmedApp.class)
@EnabledIf("false")
public class EPASContractsResourceIT {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	private static final String OFFICE_DEFAULT_ID = "1";
	//private static final String OFFICE_DEFAULT_CODEID = "225200";
	

	private static final String PERSON_DEFAULT_ID = "1";
	private static final String PERSON_DEFAULT_NAME = "Giancarlo";
	private static final String PERSON_DEFAULT_SURNAME = "Panichi";
	private static final String PERSON_DEFAULT_FISCAL_CODE = "PNCGCR75S04L103G";//"MTAGPP68D15D976W"; // "MROGLI68H29E625F";
																					// "PNCGCR75S04L103G";
	private static final String PERSON_DEFAULT_EMAIL = "giancarlo.panichi@cnr.it";
	private static final String PERSON_DEFAULT_QUALIFICATION = "6";

	private static final String CONTRACT_DEFAULT_ID = "113";
	private static final String CONTRACT_DEFAULT_BEGINDATE = "2018-12-27";
	private static final String CONTRACT_DEFAUL_ENDDATE = null;
	private static final String CONTRACT_DEFAULT_ENDCONTRACT = null;
	private static final boolean CONTRACT_DEFAULT_ONCERTIFICATE = true;
	//private static final String CONTRACT_DEFAULT_PREVIOUSCONTRACT = null;
	private static final String CONTRACT_DEFAULT_EXTERNALID = "2496";

	private static final String CONTRACT_UPDATE_ENDATE = "2021-12-31";

	@Autowired
	private MockMvc restEPASContractsMockMvc;

	@Autowired
	private EPASContractsMapper epasContractsMapper;

	@Autowired
	private Environment environment;

	private EPASContracts epasContracts;

	private EPASContractsDTO epasContractsDTO;

	/**
	 * Create a EPAS Contracts.
	 *
	 * This is a static method
	 */
	public static EPASContracts createEPASContracts() {
		EPASContracts epasContracts = new EPASContracts();
		epasContracts.setBeginDate(CONTRACT_DEFAULT_BEGINDATE);
		epasContracts.setEndDate(CONTRACT_DEFAUL_ENDDATE);
		epasContracts.setEndContract(CONTRACT_DEFAULT_ENDCONTRACT);
		epasContracts.setOnCertificate(CONTRACT_DEFAULT_ONCERTIFICATE);
		epasContracts.setPerson(createEPASPerson());
		epasContracts.setExternalId(CONTRACT_DEFAULT_EXTERNALID);
		return epasContracts;
	}

	/**
	 * Create a EPAS Persons.
	 *
	 * This is a static method
	 */
	public static EPASPersons createEPASPerson() {
		EPASPersons epasPersons = new EPASPersons();
		epasPersons.setId(PERSON_DEFAULT_ID);
		epasPersons.setName(PERSON_DEFAULT_NAME);
		epasPersons.setSurname(PERSON_DEFAULT_SURNAME);
		epasPersons.setFiscalCode(PERSON_DEFAULT_FISCAL_CODE);
		epasPersons.setEmail(PERSON_DEFAULT_EMAIL);
		epasPersons.setQualification(PERSON_DEFAULT_QUALIFICATION);
		return epasPersons;
	}

	@BeforeEach
	public void initTest() {
		for (String profileName : environment.getActiveProfiles()) {
			logger.info("Currently active profile - " + profileName);
		}
		epasContracts = createEPASContracts();
		epasContractsDTO = epasContractsMapper.epasContractsToEPASContractsDTO(epasContracts);
	}

	@Test
	public void getContractById() throws Exception {
		restEPASContractsMockMvc.perform(get("/api/epas/contracts/" + CONTRACT_DEFAULT_ID)).andExpect(status().isOk());
	}

	@Test
	public void getContractByPersonId() throws Exception {
		restEPASContractsMockMvc.perform(get("/api/epas/contracts/byPerson?id=" + PERSON_DEFAULT_ID))
				.andExpect(status().isOk());
	}

	@Test
	public void getContractByPersonFiscalcode() throws Exception {
		restEPASContractsMockMvc.perform(get("/api/epas/contracts/byPerson?fiscalCode=" + PERSON_DEFAULT_FISCAL_CODE))
				.andExpect(status().isOk());
	}

	@Test
	public void getContractByPersonEmail() throws Exception {
		restEPASContractsMockMvc.perform(get("/api/epas/contracts/byPerson?email=" + PERSON_DEFAULT_EMAIL))
				.andExpect(status().isOk());
	}

	@Test
	public void getContractMap() throws Exception {
		MvcResult result = restEPASContractsMockMvc
				.perform(get("/api/epas/contracts/map?officeId=" + OFFICE_DEFAULT_ID))
				.andExpect(status().isOk()).andReturn();

		ObjectMapper mapper = new ObjectMapper();
		LinkedHashMap<String, EPASContracts> map = mapper.readValue(result.getResponse().getContentAsString(),
				new TypeReference<LinkedHashMap<String, EPASContracts>>() {
				});
		for (String key : map.keySet()) {
			EPASContracts contract = map.get(key);
			if (contract == null) {
				logger.info("PersonId: {} has last contract null.", key);
			}
		}

	}

	@Test
	public void createContract() throws Exception {
		restEPASContractsMockMvc
				.perform(post("/api/epas/contracts").contentType(MediaType.APPLICATION_JSON)
						.content(TestUtil.convertObjectToJsonBytes(epasContractsDTO)).with(csrf()))
				.andExpect(status().isCreated());
	}

	@Test
	public void updateContractById() throws Exception {
		epasContractsDTO.setEndDate(CONTRACT_UPDATE_ENDATE);
		MvcResult mvcResult = restEPASContractsMockMvc
				.perform(put("/api/epas/contracts/" + CONTRACT_DEFAULT_ID).contentType(MediaType.APPLICATION_JSON)
						.content(TestUtil.convertObjectToJsonBytes(epasContractsDTO)).with(csrf()))
				.andExpect(status().isOk()).andReturn();
		ObjectMapper mapper = new ObjectMapper();

		EPASContracts epasContracts = mapper.readValue(mvcResult.getResponse().getContentAsByteArray(),
				EPASContracts.class);
		logger.info("Updated EPAS Contract by id: " + epasContracts);
	}

	@Test
	public void setEPASContractPrevious() throws Exception {
		restEPASContractsMockMvc
				.perform(delete("/api/epas/contracts/setPreviousContract?id=" + CONTRACT_DEFAULT_ID).with(csrf()))
				.andExpect(status().is2xxSuccessful());
		logger.info("Set to Previous EPAS Contract by id: " + CONTRACT_DEFAULT_ID);
	}

	@Test
	public void setEPASContractUnsetPrevious() throws Exception {
		restEPASContractsMockMvc
				.perform(delete("/api/epas/contracts/unsetPreviousContract?id=" + CONTRACT_DEFAULT_ID).with(csrf()))
				.andExpect(status().is2xxSuccessful());
		logger.info("Set to Previous EPAS Contract by id: " + CONTRACT_DEFAULT_ID);
	}

	@Test
	public void deleteContractById() throws Exception {
		restEPASContractsMockMvc.perform(delete("/api/epas/contracts/" + CONTRACT_DEFAULT_ID).with(csrf()))
				.andExpect(status().is2xxSuccessful());
		logger.info("Deleted EPAS Contract by id: " + CONTRACT_DEFAULT_ID);
	}

	@Test
	public void istiUpdateContract() throws Exception {
		String userDirectory = System.getProperty("user.dir");
		logger.info(userDirectory);
		List<EPASContracts> istiContracts = null;
		try (Stream<String> stream = Files
				.lines(Paths.get("src/test/resources/it/cnr/isti/epasmed/web/rest/epas/posizioniISTI.csv"))) {
			istiContracts = stream.skip(1).map(istiMapToContract).collect(Collectors.toList());
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage(), e);
			return;
		}
		logger.info("ISTI Contracts loaded");

		for (EPASContracts c : istiContracts) {
			logger.info("Contract: {}", c);
		}

		List<EPASContracts> enrichedISTIContracts = new ArrayList<>();
		for (EPASContracts c : istiContracts) {
			if (c.getPerson() != null && c.getPerson().getFiscalCode() != null
					&& !c.getPerson().getFiscalCode().isEmpty()) {
				c.setPerson(retrievePersonByFiscalCode(c.getPerson().getFiscalCode()));
				enrichedISTIContracts.add(c);
			}
		}

		logger.info("ISTI enriched contracts with persons");

		List<EPASContracts> updatableContracts = new ArrayList<>();
		for (EPASContracts ec : enrichedISTIContracts) {
			if (ec.getPerson() != null && ec.getPerson().getFiscalCode() != null
					&& !ec.getPerson().getFiscalCode().isEmpty()) {
				List<EPASContracts> personContract = retrieveContractsByFiscalCode(ec);
				for (EPASContracts persCont : personContract) {
					if (persCont.getBeginDate().compareTo(ec.getBeginDate()) == 0) {
						persCont.setExternalId(ec.getExternalId());
						updatableContracts.add(persCont);
						break;
					}
				}
			}

		}

		logger.info("ISTI Updateble contracts created");

		List<EPASContractsDTO> updatableContractsDTO = epasContractsMapper
				.epasContractsToEPASContractsDTOs(updatableContracts);

		for (EPASContractsDTO u : updatableContractsDTO) {
			logger.info("Contract Updatable: {}", u);
			restEPASContractsMockMvc.perform(put("/api/epas/contracts/" + u.getId())
					.contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(u)).with(csrf()))
					.andExpect(status().isOk());

		}
		logger.info("ISTI Update contracts end");

	}

	private List<EPASContracts> retrieveContractsByFiscalCode(EPASContracts ec)
			throws Exception, JsonProcessingException, JsonMappingException, UnsupportedEncodingException {
		MvcResult result = restEPASContractsMockMvc
				.perform(get("/api/epas/contracts/byPerson?fiscalCode=" + ec.getPerson().getFiscalCode()))
				.andExpect(status().isOk()).andReturn();
		ObjectMapper mapper = new ObjectMapper();
		List<EPASContracts> epasContractsForPerson = mapper.readValue(result.getResponse().getContentAsString(),
				new TypeReference<List<EPASContracts>>() {
				});
		return epasContractsForPerson;
	}

	private EPASPersons retrievePersonByFiscalCode(String fiscalCode) throws Exception {
		MvcResult result = restEPASContractsMockMvc.perform(get("/api/epas/persons/show?fiscalCode=" + fiscalCode))
				.andExpect(status().isOk()).andReturn();

		ObjectMapper mapper = new ObjectMapper();
		EPASPersons epasPersons = mapper.readValue(result.getResponse().getContentAsString(),
				new TypeReference<EPASPersons>() {
				});
		return epasPersons;
	}

	private Function<String, EPASContracts> istiMapToContract = (line) -> {

		String[] c = line.split(",");// a CSV has comma separated lines

		EPASContracts contract = new EPASContracts();

		// <-- this is the first column in the csv file
		if (c[0] != null && c[0].length() > 0) {
			String externalId = c[0].substring(1, c[0].length() - 1);
			if (externalId != null && !externalId.isEmpty()) {
				contract.setExternalId(externalId);
			}
		}

		if (c[1] != null && c[1].length() > 0) {
			String beginDate = c[1].substring(1, c[1].length() - 1);
			if (beginDate != null && !beginDate.isEmpty()) {
				contract.setBeginDate(beginDate);
			}
		}

		if (c[2] != null && c[2].length() > 0) {
			String endDate = c[2].substring(1, c[2].length() - 1);
			if (endDate != null && !endDate.isEmpty()) {
				contract.setEndDate(endDate);
			}
		}

		EPASPersons ep = new EPASPersons();
		if (c[4] != null && c[4].length() > 0) {
			String fiscalcode = c[4].substring(1, c[4].length() - 1);
			if (fiscalcode != null && !fiscalcode.isEmpty()) {
				ep.setFiscalCode(fiscalcode);
			}
		}

		if (c[5] != null && c[5].length() > 0) {
			String surname = c[5].substring(1, c[5].length() - 1);
			if (surname != null && !surname.isEmpty()) {
				ep.setSurname(surname);
			}
		}

		if (c[6] != null && c[6].length() > 0) {
			String name = c[6].substring(1, c[6].length() - 1);
			if (name != null && !name.isEmpty()) {
				ep.setName(name);
			}
		}

		contract.setPerson(ep);

		return contract;
	};

}
