package it.cnr.isti.epasmed.web.rest.epas;

import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.util.ArrayList;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.EnabledIf;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import it.cnr.isti.epasmed.EpasmedApp;
import it.cnr.isti.epasmed.epas.dto.EPASGroupsDTO;
import it.cnr.isti.epasmed.epas.mapper.EPASGroupsMapper;
import it.cnr.isti.epasmed.epas.model.EPASGroups;
import it.cnr.isti.epasmed.epas.model.EPASGroupsManager;
import it.cnr.isti.epasmed.epas.model.EPASOffice;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;
import it.cnr.isti.epasmed.web.rest.TestUtil;

/**
 * Integration tests for the {@link EPASGroupsResource} REST controller.
 */
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@SpringBootTest(classes = EpasmedApp.class)
@EnabledIf("false")
public class EPASGroupsResourceIT {

	private final Logger log = LoggerFactory.getLogger(getClass());

	private static final String OFFICE_DEFAULT_ID = "1";
	private static final String OFFICE_DEFAULT_NAME = "ISTI - Pisa";
	private static final String OFFICE_DEFAULT_CODE = "074000";
	private static final String OFFICE_DEFAULT_CODEID = "225200";

	private static final String MANAGER_DEFAULT_ID = "1";
	private static final String MANAGER_DEFAULT_FULLNAME = "Giancarlo Panichi";
	private static final String MANAGER_DEFAULT_FISCALCODE = "PNCGCRL75S04L103G";
	private static final String MANAGER_DEFAULT_EMAIL = "giancarlo.panichi@cnr.it";
	private static final String MANAGER_DEFULT_EPPN = "giancarlo.panichi@cnr.it";
	private static final String MANAGER_DEFAULT_NUMBER = "15409";

	private static final String GROUP_DEFAULT_ID = "1";
	private static final String GROUP_DEFAULT_NAME = "GruppoTestRest";
	private static final String GROUP_DEFAULT_DESCRIPTION = "Gruppo di Test Rest";
	private static final String GROUP_DEFAUL_ENDDATE = null;
	private static final String GROUP_DEFAULT_EXTERNALID = "1";
	private static final String GROUP_DEFAULT_UPDATEDAT = null;

	private static final String GROUP_UPDATED_DESCRIPTION = "Gruppo di Test Updated";

	private ArrayList<EPASGroupsDTO> epasGroupsDTOList;

	@Autowired
	private MockMvc restEPASGroupsMockMvc;

	@Autowired
	private EPASGroupsMapper epasGroupsMapper;

	@Autowired
	private Environment environment;

	private EPASGroups epasGroups;

	private EPASGroupsDTO epasGroupsDTO;

	/**
	 * Create a EPAS Groups.
	 *
	 * This is a static method
	 */
	public static EPASGroups createEPASGroups() {
		EPASGroups epasGroups = new EPASGroups();
		epasGroups.setName(GROUP_DEFAULT_NAME);
		epasGroups.setDescription(GROUP_DEFAULT_DESCRIPTION);
		epasGroups.setEndDate(GROUP_DEFAUL_ENDDATE);
		epasGroups.setExternalId(GROUP_DEFAULT_EXTERNALID);
		epasGroups.setManager(createManager());
		epasGroups.setUpdatedAt(GROUP_DEFAULT_UPDATEDAT);
		epasGroups.setOffice(createOffice());
		return epasGroups;
	}

	private static EPASGroupsManager createManager() {
		EPASGroupsManager epasGroupsManager = new EPASGroupsManager();
		epasGroupsManager.setId(MANAGER_DEFAULT_ID);
		epasGroupsManager.setFullname(MANAGER_DEFAULT_FULLNAME);
		epasGroupsManager.setFiscalCode(MANAGER_DEFAULT_FISCALCODE);
		epasGroupsManager.setEmail(MANAGER_DEFAULT_EMAIL);
		epasGroupsManager.setEppn(MANAGER_DEFULT_EPPN);
		epasGroupsManager.setNumber(MANAGER_DEFAULT_NUMBER);
		return epasGroupsManager;
	}

	public static EPASOffice createOffice() {
		EPASOffice epasOffice = new EPASOffice();
		epasOffice.setId(OFFICE_DEFAULT_ID);
		epasOffice.setName(OFFICE_DEFAULT_NAME);
		epasOffice.setCode(OFFICE_DEFAULT_CODE);
		epasOffice.setCodeId(OFFICE_DEFAULT_CODEID);
		return epasOffice;
	}

	private ArrayList<EPASGroupsDTO> createEPASGroupsDTOList() {
		epasGroupsDTOList = new ArrayList<>();
		epasGroupsDTOList.add(new EPASGroupsDTO("AREA", null, "1", null, "1", "AREA", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Consiglio di Istituto", null, "2", null, "1", "CI", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Dependable Computing", null, "3", null, "1", "DC", "1"));
		epasGroupsDTOList
				.add(new EPASGroupsDTO("Formal Methods and Tools", null, "4", null, "1", "FMT", "1"));
		epasGroupsDTOList
				.add(new EPASGroupsDTO("Sistema Informativo", null, "5", null, "1", "GL - SI", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Domotics", null, "6", null, "1", "HA", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Human Interfaces in Information Systems", null, "7", null, "1", "HIIS",
				"1"));
		epasGroupsDTOList
				.add(new EPASGroupsDTO("High Performance Computing", null, "8", null, "1", "HPC", "1"));
		epasGroupsDTOList.add(
				new EPASGroupsDTO("Knowledge Discovery and Data Mining", null, "9", null, "1", "KDD", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Mechanics of Materials and Structures", null, "10", null, "1", "MMS",
				"1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Networked Multimedia Information System", null, "11", null, "1",
				"NeMIS", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Servizi Internet", null, "12", null, "1", "NS", "1"));
		epasGroupsDTOList
				.add(new EPASGroupsDTO("Servizio Amministrazione", null, "13", null, "1", "SA", "1"));
		epasGroupsDTOList
				.add(new EPASGroupsDTO("Servizio Attività Generali", null, "14", null, "1", "SAG", "1"));
		epasGroupsDTOList
				.add(new EPASGroupsDTO("Servizio Attività Logistiche", null, "15", null, "1", "SAL", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Servizio Attività Scientifiche - Progetti e Contratti", null, "16",
				null, "1", "SAS - PC", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Servizio Attività Scientifiche - Segreteria Scientifica", null, "17",
				null, "1", "SAS - SS", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Servizio Biblioteca e Centro di Documentazione Scientifica", null,
				"18", null, "1", "SB", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Software Engineering", null, "19", null, "1", "SE", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Software Engineering & Dependable Computing", null, "20", null, "1",
				"SEDC", "1"));
		epasGroupsDTOList
				.add(new EPASGroupsDTO("Space Flight Dynamics", null, "21", null, "1", "SFD", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Signals and Images", null, "22", null, "1", "SI", "1"));
		epasGroupsDTOList.add(
				new EPASGroupsDTO("System and Software Evaluation", null, "23", null, "1", "SSE", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Servizio Supporto Informatico e Formazione - Formazione e Sviluppo",
				null, "24", null, "1", "SSIF - FS", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Servizio Supporto Informatico e Formazione - Supporto Informatico",
				null, "25", null, "1", "SSIF - SI", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Visual Computing", null, "26", null, "1", "VC", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Wireless Networks", null, "27", null, "1", "WN", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Sistemi di Rete e Servizi Internet", null, "28", null, "1", "SRSI",
				"1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Servizio Attività Scientifiche - Gestione Privacy", null, "29", null,
				"1", "SAS - GP", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Infrastruttura Informatica D4Science", null, "30", null, "1",
				"GL - D4Science", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Servizio Infrastruttura Informatica ISTI e Supporto ai Servizi", null,
				"31", null, "1", "S2I2S", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Artificial Intelligence for Media and Humanities", null, "32", null,
				"1", "AIMH", "1"));
		epasGroupsDTOList.add(new EPASGroupsDTO("Infrastructures for Science", null, "33", null, "1", "InfraScience",
				"1"));
		return epasGroupsDTOList;
	}

	@BeforeEach
	public void initTest() {
		for (String profileName : environment.getActiveProfiles()) {
			log.info("Currently active profile - " + profileName);
		}
		epasGroups = createEPASGroups();
		epasGroupsDTO = epasGroupsMapper.epasGroupsToEPASGroupsDTO(epasGroups);
		epasGroupsDTOList = createEPASGroupsDTOList();
	}

	@Test
	public void getGroup() throws Exception {
		restEPASGroupsMockMvc.perform(get("/api/epas/groups/" + 9)).andExpect(status().isOk());
	}

	@Test
	public void getGroupList() throws Exception {
		restEPASGroupsMockMvc.perform(get("/api/epas/groups?officeId=" + OFFICE_DEFAULT_ID)).andExpect(status().isOk());
	}

	@Test
	public void createGroup() throws Exception {
		restEPASGroupsMockMvc
				.perform(post("/api/epas/groups").contentType(MediaType.APPLICATION_JSON)
						.content(TestUtil.convertObjectToJsonBytes(epasGroupsDTO)).with(csrf()))
				.andExpect(status().isCreated());
	}

	@Test
	public void istiCreateGroups() throws Exception {
		for (EPASGroupsDTO istiGroup : epasGroupsDTOList) {
			restEPASGroupsMockMvc
					.perform(post("/api/epas/groups").contentType(MediaType.APPLICATION_JSON)
							.content(TestUtil.convertObjectToJsonBytes(istiGroup)).with(csrf()))
					.andExpect(status().isCreated());
		}
	}

	@Test
	public void istiUpdateGroups() throws Exception {
		MvcResult result = restEPASGroupsMockMvc.perform(get("/api/epas/groups?officeId=" + OFFICE_DEFAULT_ID))
				.andExpect(status().isOk()).andReturn();
		ObjectMapper mapper = new ObjectMapper();
		List<EPASGroups> epasG = mapper.readValue(result.getResponse().getContentAsString(),
				new TypeReference<List<EPASGroups>>() {
				});

		log.info("EPASGroups: {}", epasG.size());

		List<EPASGroupsDTO> epasGDTO = epasGroupsMapper.epasGroupsToEPASGroupsDTOs(epasG);

		istiEnrichEPASGroupData(epasGDTO);

		for (EPASGroupsDTO g : epasGDTO) {
			log.info("Updated EPASGroup: {}", g);
		}

		for (EPASGroupsDTO g : epasGDTO) {
			restEPASGroupsMockMvc.perform(put("/api/epas/groups/" + g.getId()).contentType(MediaType.APPLICATION_JSON)
					.content(TestUtil.convertObjectToJsonBytes(g)).with(csrf())).andExpect(status().isOk());
		}
		log.info("EPASGroups ISTI Updated");
		
	}

	private void istiEnrichEPASGroupData(List<EPASGroupsDTO> epasGDTO) {
		for (EPASGroupsDTO g : epasGDTO) {
			for (EPASGroupsDTO m : epasGroupsDTOList) {
				if (m.getName().compareTo(g.getName()) == 0) {
					g.setExternalId(m.getExternalId());
					g.setOfficeId("1");
					break;
				}
			}
		}

	}
	
	@Test
	public void getGroupByExternalId() throws Exception{
		String externalId="29";
		MvcResult mvcResult=restEPASGroupsMockMvc.perform(get("/api/epas/groups?officeId=" + OFFICE_DEFAULT_ID)).andExpect(status().isOk()).andReturn();
		ObjectMapper mapper = new ObjectMapper();

		List<EPASGroups> epasGroups = mapper.readValue(mvcResult.getResponse().getContentAsString(),
				new TypeReference<List<EPASGroups>>() {
				});	
		for(EPASGroups g:epasGroups) {
			if(g.getExternalId()!=null&&!g.getExternalId().isEmpty()&&
					g.getExternalId().compareTo(externalId)==0) {
				log.info("Found Group: {}",g);
				break;
			}
		}
		
	}

	@Test
	public void updateGroupById() throws Exception {
		epasGroupsDTO.setDescription(GROUP_UPDATED_DESCRIPTION);
		MvcResult mvcResult = restEPASGroupsMockMvc
				.perform(put("/api/epas/groups/" + GROUP_DEFAULT_ID).contentType(MediaType.APPLICATION_JSON)
						.content(TestUtil.convertObjectToJsonBytes(epasGroupsDTO)).with(csrf()))
				.andExpect(status().isOk()).andReturn();
		ObjectMapper mapper = new ObjectMapper();

		EPASGroups epasGroups = mapper.readValue(mvcResult.getResponse().getContentAsByteArray(), EPASGroups.class);
		log.info("Updated EPAS Group by id: " + epasGroups);
	}

	@Test
	public void deleteGroupById() throws Exception {
		restEPASGroupsMockMvc.perform(delete("/api/epas/groups/" + GROUP_DEFAULT_ID).with(csrf()))
				.andExpect(status().is2xxSuccessful());
		log.info("Deleted EPAS Group by id: " + GROUP_DEFAULT_ID);
	}

}
