package it.cnr.isti.epasmed.web.rest.epas;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.env.Environment;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.EnabledIf;
import org.springframework.test.web.servlet.MockMvc;

import it.cnr.isti.epasmed.EpasmedApp;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;

/**
 * Integration tests for the {@link EPASLeavesResource} REST controller.
 */
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@SpringBootTest(classes = EpasmedApp.class)
@EnabledIf("false")
public class EPASLeavesResourceIT {

	private final Logger log = LoggerFactory.getLogger(getClass());

	private static final String OFFICE_DEFAULT_ID = "1";
	// private static final String OFFICE_DEFAULT_NAME = "ISTI - Pisa";
	// private static final String OFFICE_DEFAULT_CODE = "074000";
	// private static final String OFFICE_DEFAULT_CODEID = "225200";
	private static final String YEAR_DEFAULT = "2022";
	private static final String INCLUDEDETAILS_DEFAULT = "false";

	private static final String PERSON_DEFAULT_ID = "1";
	// private static final String PERSON_DEFAULT_NAME = "Giulio";
	// private static final String PERSON_DEFAULT_SURNAME = "Mori";
	private static final String PERSON_DEFAULT_FISCAL_CODE = "MROGLI68H29E625F";
	private static final String PERSON_DEFAULT_EMAIL = "giulio.mori@cnr.it";

	@Autowired
	private MockMvc restEPASLeavesMockMvc;

	@Autowired
	private Environment environment;

	// private EPASLeaves epasLeaves;

	@BeforeEach
	public void initTest() {
		for (String profileName : environment.getActiveProfiles()) {
			log.info("Currently active profile - " + profileName);
		}
	}

	@Test
	public void getEPASLeavesByPersonId() throws Exception {
		restEPASLeavesMockMvc.perform(get("/api/epas/leaves/byPersonAndYear?id=" + PERSON_DEFAULT_ID + "&year="
				+ YEAR_DEFAULT + "&includeDetails=" + INCLUDEDETAILS_DEFAULT)).andExpect(status().isOk());
	}

	@Test
	public void getEPASLeaveByPersonFiscalcode() throws Exception {
		restEPASLeavesMockMvc.perform(get("/api/epas/leaves/byPersonAndYear?fiscalCode=" + PERSON_DEFAULT_FISCAL_CODE
				+ "&year=" + YEAR_DEFAULT + "&includeDetails=" + INCLUDEDETAILS_DEFAULT)).andExpect(status().isOk());
	}

	@Test
	public void getEPASLeavesByPersonEmail() throws Exception {
		restEPASLeavesMockMvc.perform(get("/api/epas/leaves/byPersonAndYear?email=" + PERSON_DEFAULT_EMAIL + "&year="
				+ YEAR_DEFAULT + "&includeDetails=" + INCLUDEDETAILS_DEFAULT)).andExpect(status().isOk());
	}

	@Test
	public void getEPASLeavesByOfficeId() throws Exception {
		restEPASLeavesMockMvc.perform(get("/api/epas/leaves/byOfficeAndYear?officeId=" + OFFICE_DEFAULT_ID + "&year="
				+ YEAR_DEFAULT + "&includeDetails=" + INCLUDEDETAILS_DEFAULT)).andExpect(status().isOk());
	}

}
