package it.cnr.isti.epasmed.web.rest.epas;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.env.Environment;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.EnabledIf;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import it.cnr.isti.epasmed.EpasmedApp;
import it.cnr.isti.epasmed.epas.dto.EPASPersonWorkingTimeDTO;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;

/**
 * Integration tests for the {@link EPASPersonWorkingTimeResource} REST
 * controller.
 */
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@SpringBootTest(classes = EpasmedApp.class)
@EnabledIf("false")
public class EPASPersonWorkingTimeIT {

	private static final Logger logger = LoggerFactory.getLogger(EPASPersonWorkingTimeIT.class);

	private static final String OFFICE_DEFAULT_ID = "1";
	// private static final String OFFICE_DEFAULT_NAME = "ISTI - Pisa";
	// private static final String OFFICE_DEFAULT_CODE = "074000";
	// private static final String OFFICE_DEFAULT_CODEID = "225200";

	private static final String PERSON_DEFAULT_ID = "78";
	//private static final String PERSON_DEFAULT_FISCAL_CODE = "MROGLI68H29E625F";
	private static final String PERSON_DEFAULT_FISCAL_CODE = "PNCGCR75S04L103G";
	//private static final String PERSON_DEFAULT_FISCAL_CODE = "MLSDNL59T56E281S";
	//private static final String PERSON_DEFAULT_FISCAL_CODE = "PCCMRA63M50G702W";
	//private static final String PERSON_DEFAULT_FISCAL_CODE = "LMBGNN59T09G702K";
	//private static final String PERSON_DEFAULT_FISCAL_CODE = "MNGPLA70T22D612X";
	
	
	@Autowired
	private MockMvc restEPASPersonWorkingTimeMockMvc;

	@Autowired
	private Environment environment;

	@BeforeEach
	public void initTest() {
		for (String profileName : environment.getActiveProfiles()) {
			logger.info("Currently active profile - " + profileName);
		}
	}

	@Test
	public void getPersonWorkingTimeByPersonId() throws Exception {
		MvcResult result = restEPASPersonWorkingTimeMockMvc
				.perform(get("/api/epas/personworkingtime/byPerson?id=" + PERSON_DEFAULT_ID)).andExpect(status().isOk())
				.andReturn();
		ObjectMapper mapper = new ObjectMapper();
		EPASPersonWorkingTimeDTO epasWTT = mapper.readValue(result.getResponse().getContentAsString(),
				new TypeReference<EPASPersonWorkingTimeDTO>() {
				});
		logger.info("EPASPersonWorkingTime: {}", epasWTT);

	}

	@Test
	public void getPersonWorkingTimeByPersonFiscalCode() throws Exception {
		MvcResult result = restEPASPersonWorkingTimeMockMvc
				.perform(get("/api/epas/personworkingtime/byPerson?fiscalCode=" + PERSON_DEFAULT_FISCAL_CODE))
				.andExpect(status().isOk()).andReturn();
		ObjectMapper mapper = new ObjectMapper();
		EPASPersonWorkingTimeDTO epasWTT = mapper.readValue(result.getResponse().getContentAsString(),
				new TypeReference<EPASPersonWorkingTimeDTO>() {
				});
		logger.info("EPASPersonWorkingTime: {}", epasWTT);

	}
	
	
	@Test
	public void getPersonWorkingTimeList() throws Exception {
		MvcResult result = restEPASPersonWorkingTimeMockMvc
				.perform(get("/api/epas/personworkingtime/list?officeId=" + OFFICE_DEFAULT_ID)).andExpect(status().isOk())
				.andReturn();
		ObjectMapper mapper = new ObjectMapper();
		List<EPASPersonWorkingTimeDTO> epasWTTList = mapper.readValue(result.getResponse().getContentAsString(),
				new TypeReference<List<EPASPersonWorkingTimeDTO>>() {
				});
		logger.info("EPASPersonWorkingTime List: {}", epasWTTList);

	}

	

}
