package it.cnr.isti.epasmed.web.rest.epas;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.env.Environment;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.EnabledIf;
import org.springframework.test.web.servlet.MockMvc;

import it.cnr.isti.epasmed.EpasmedApp;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;

/**
 * Integration tests for the {@link EPASTimeCardsResource} REST controller.
 */
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@SpringBootTest(classes = EpasmedApp.class)
@EnabledIf("false")
public class EPASTimeCardsResourceIT {

	private final Logger log = LoggerFactory.getLogger(getClass());
	
	//private static final String OFFICE_DEFAULT_ID = "1";
	//private static final String OFFICE_DEFAULT_NAME = "ISTI - Pisa";
	//private static final String OFFICE_DEFAULT_CODE = "074000";
	private static final String OFFICE_DEFAULT_CODEID = "225200";

	private static final String PERSON_DEFAULT_ID = "113";
	//private static final String PERSON_DEFAULT_NAME = "Giuseppe";
	//private static final String PERSON_DEFAULT_SURNAME = "Amato";
	//private static final String PERSON_DEFAULT_FISCAL_CODE = "MTAGPP68D15D976W";
	private static final String PERSON_DEFAULT_FISCAL_CODE = "BRTNTN60R56E974Z";

	
	private static final String PERSON_DEFAULT_EMAIL = "giuseppe.amato@cnr.it";
	//private static final String PERSON_DEFAULT_QUALIFICATION = "2";
	
	@Autowired
	private MockMvc restEPASTimeCardsMockMvc;

	@Autowired
	private Environment environment;

	@BeforeEach
	public void initTest() {
		for (String profileName : environment.getActiveProfiles()) {
			log.info("Currently active profile - " + profileName);
		}
		log.info("System env - " + System.getenv("spring.profiles.active"));
	}

	@Test
	public void getTimeCardByPersonId() throws Exception {
		restEPASTimeCardsMockMvc.perform(get("/api/epas/timecards/byPerson?id="+PERSON_DEFAULT_ID+"&year=2022&month=6")).andExpect(status().isOk());
	}
	
	@Test
	public void getTimeCardByPersonFiscalCode() throws Exception {
		restEPASTimeCardsMockMvc.perform(get("/api/epas/timecards/byPerson?fiscalCode="+PERSON_DEFAULT_FISCAL_CODE+"&year=2022&month=6")).andExpect(status().isOk());
	}
	
	@Test
	public void getTimeCardByPersonEmail() throws Exception {
		restEPASTimeCardsMockMvc.perform(get("/api/epas/timecards/byPerson?email="+PERSON_DEFAULT_EMAIL+"&year=2022&month=6")).andExpect(status().isOk());
	}
	
	@Test
	public void getTimeCardByOfficeCodeId() throws Exception {
		restEPASTimeCardsMockMvc.perform(get("/api/epas/timecards/byOffice?codeId="+OFFICE_DEFAULT_CODEID+"&year=2022&month=6")).andExpect(status().isOk());
	}
	
}
