package it.cnr.isti.epasmed.web.rest.epas;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.env.Environment;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.EnabledIf;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import it.cnr.isti.epasmed.EpasmedApp;
import it.cnr.isti.epasmed.epas.model.EPASValidates;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;

/**
 * Integration tests for the {@link EPASValidatesResource} REST
 * controller.
 */
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@SpringBootTest(classes = EpasmedApp.class)
@EnabledIf("false")
public class EPASValidatesResourceIT {

	private final Logger log = LoggerFactory.getLogger(getClass());

	//private static final String OFFICE_DEFAULT_ID = "1";
	// private static final String OFFICE_DEFAULT_NAME = "ISTI - Pisa";
	// private static final String OFFICE_DEFAULT_CODE = "074000";
	private static final String OFFICE_DEFAULT_CODEID = "225200";

	@Autowired
	private MockMvc restEPASValidatesMockMvc;

	@Autowired
	private Environment environment;

	@BeforeEach
	public void initTest() {
		for (String profileName : environment.getActiveProfiles()) {
			log.info("Currently active profile - " + profileName);
		}
	}

	@Test
	public void isValidatesByPersonEmail() throws Exception {
		MvcResult result = restEPASValidatesMockMvc.perform(get("/api/epas/validates/is?email=giancarlo.panichi@cnr.it&"
				+ "year=2021&month=11")).andExpect(status().isOk()).andReturn();
		ObjectMapper mapper = new ObjectMapper();
		String validated = mapper.readValue(result.getResponse().getContentAsString(),
				new TypeReference<String>() {
				});
		log.info("EPASValidates is: {}", validated);
	}
	
	@Test
	public void getEPASValidatesByOfficeCodeId() throws Exception {
		MvcResult result = restEPASValidatesMockMvc.perform(get("/api/epas/validates?officeCodeId="+OFFICE_DEFAULT_CODEID+"&"
				+ "year=2021&month=11")).andExpect(status().isOk()).andReturn();
		ObjectMapper mapper = new ObjectMapper();
		EPASValidates validates = mapper.readValue(result.getResponse().getContentAsString(),
				new TypeReference<EPASValidates>() {
				});
		log.info("EPASValidates: {}", validates);
	
	}
	
	
}
