package it.cnr.isti.epasmed.web.rest.epas;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.env.Environment;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.EnabledIf;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import it.cnr.isti.epasmed.EpasmedApp;
import it.cnr.isti.epasmed.epas.model.EPASWorkingTimeTypeDays;
import it.cnr.isti.epasmed.epas.model.EPASWorkingTimeTypes;
import it.cnr.isti.epasmed.security.AuthoritiesConstants;

/**
 * Integration tests for the {@link EPASWorkingTimeTypesResource} REST
 * controller.
 */
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@SpringBootTest(classes = EpasmedApp.class)
@EnabledIf("false")
public class EPASWorkingTimeTypesResourceIT {

	private static final Logger logger = LoggerFactory.getLogger(EPASWorkingTimeTypesResourceIT.class);

	// private static final String OFFICE_DEFAULT_ID = "1";
	// private static final String OFFICE_DEFAULT_NAME = "ISTI - Pisa";
	// private static final String OFFICE_DEFAULT_CODE = "074000";
	private static final String OFFICE_DEFAULT_CODEID = "225200";

	private static final String DEFAULT_WORKING_TIME_TYPE_ID = "1";

	@Autowired
	private MockMvc restEPASWorkingTimeTypesMockMvc;

	@Autowired
	private Environment environment;

	@BeforeEach
	public void initTest() {
		for (String profileName : environment.getActiveProfiles()) {
			logger.info("Currently active profile - " + profileName);
		}
	}

	@Test
	public void getWorkingTimeTypesById() throws Exception {
		MvcResult result = restEPASWorkingTimeTypesMockMvc
				.perform(get("/api/epas/workingtimetypes/" + DEFAULT_WORKING_TIME_TYPE_ID)).andExpect(status().isOk())
				.andReturn();
		ObjectMapper mapper = new ObjectMapper();
		EPASWorkingTimeTypes epasWTT = mapper.readValue(result.getResponse().getContentAsString(),
				new TypeReference<EPASWorkingTimeTypes>() {
				});
		logger.info("EPASWorkingTimeTypes: {}", epasWTT.getDescription());
		for (EPASWorkingTimeTypeDays d : epasWTT.getWorkingTimeTypeDays()) {
			logger.info("EPAS WorkingTimeTypeDay: {}", d);
		}
	}

	@Test
	public void getWorkingTimeTypesList() throws Exception {
		MvcResult result = restEPASWorkingTimeTypesMockMvc
				.perform(get("/api/epas/workingtimetypes?officeCodeId=" + OFFICE_DEFAULT_CODEID))
				.andExpect(status().isOk()).andReturn();

		ObjectMapper mapper = new ObjectMapper();
		List<EPASWorkingTimeTypes> epasWTTList = mapper.readValue(result.getResponse().getContentAsString(),
				new TypeReference<List<EPASWorkingTimeTypes>>() {
				});

		logger.info("EPASWorkingTimeTypes: {}", epasWTTList.size());
		for (EPASWorkingTimeTypes o : epasWTTList) {
			logger.info("EPAS WorkingTimeType: {}", o);
		}

	}

}
